// Reset

.@{prefix}-container, .@{prefix}-container *, .@{prefix}-widget, .@{prefix}-widget *, .@{prefix}-reset {
	margin: 0; padding: 0; border: 0; outline: 0;
	vertical-align: top; background: transparent;
	text-decoration: none; color: @text;
	font-family: @font-family;
	font-size: @font-size; text-shadow: none; float: none;
	position: static; width: auto; height: auto;
	white-space: nowrap; cursor: inherit;
	-webkit-tap-highlight-color: transparent;
	line-height: normal; font-weight: normal;
	text-align: left;
	-moz-box-sizing: content-box;
	-webkit-box-sizing: content-box;
	box-sizing: content-box;
	direction: ltr;
	max-width: none;
}

.@{prefix}-widget button {
	-moz-box-sizing: border-box;
	-webkit-box-sizing: border-box;
	box-sizing: border-box;
}

.@{prefix}-container *[unselectable] {
	-moz-user-select: none;
	-webkit-user-select: none;
	-o-user-select: none;
	user-select: none;
}
