tinymce.addI18n('cs', {
"Paste Formatting Options": "Mo\u017Enosti form\u00E1tov\u00E1n\u00ED p\u0159i vkl\u00E1d\u00E1n\u00ED",
"Choose to keep or remove formatting in the pasted content.": "Rozhodn\u011Bte se, zda chcete zachovat nebo odebrat form\u00E1tov\u00E1n\u00ED vkl\u00E1dan\u00E9ho obsahu.",
"Keep Formatting": "Zachovat form\u00E1tov\u00E1n\u00ED",
"Remove Formatting": "Odebrat form\u00E1tov\u00E1n\u00ED",
"Local Image Import": "Import m\u00EDstn\u00EDho obr\u00E1zku",
"Trigger paste again from the keyboard to paste content with images.": "K vlo\u017Een\u00ED obsahu s obr\u00E1zky aktivovat pomoc\u00ED kl\u00E1vesnice op\u011Btovn\u00E9 vlo\u017Een\u00ED.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Pro import obr\u00E1zk\u016F z Microsoft Office je zapot\u0159eb\u00ED Adobe Flash. Instalujte <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Stiskn\u011Bte kl\u00E1vesu <span class=\"ephox-polish-help-kbd\">ESC</span> k ignorov\u00E1n\u00ED m\u00EDstn\u00EDch obr\u00E1zk\u016F a pokra\u010Dujte v \u00FAprav\u00E1ch.",
"Please wait...": "\u010Cekejte pros\u00EDm...",
"Your browser security settings may be preventing images from being imported.": "Nastaven\u00ED zabezpe\u010Den\u00ED va\u0161eho prohl\u00ED\u017Ee\u010De pravd\u011Bpodobn\u011B br\u00E1n\u00ED importu obr\u00E1zk\u016F.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Nastaven\u00ED zabezpe\u010Den\u00ED va\u0161eho prohl\u00ED\u017Ee\u010De pravd\u011Bpodobn\u011B br\u00E1n\u00ED importu obr\u00E1zk\u016F. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Dal\u0161\u00ED informace o vkl\u00E1d\u00E1n\u00ED v Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari nepodporuje p\u0159\u00EDm\u00E9 vkl\u00E1d\u00E1n\u00ED obr\u00E1zk\u016F. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Dal\u0161\u00ED informace o vkl\u00E1d\u00E1n\u00ED obr\u00E1zk\u016F v Safari</a>",
"The images service was not found: (": "Slu\u017Eba pro pr\u00E1ci s obr\u00E1zky nebyla nalezena: (",
"Image failed to upload: (": "Obr\u00E1zek se nepoda\u0159ilo odeslat: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Vkl\u00E1d\u00E1n\u00ED m\u00EDstn\u00EDch obr\u00E1zk\u016F je zak\u00E1z\u00E1no. M\u00EDstn\u00ED obr\u00E1zky byly z vlo\u017Een\u00E9ho obsahu odstran\u011Bny."
});