tinymce.addI18n('hr', {
"Paste Formatting Options": "Zalijepi mogu\u0107nosti oblikovanja",
"Choose to keep or remove formatting in the pasted content.": "Izaberite ho\u0107ete li zadr\u017Eati ili ukloniti oblikovanje u zalijepljenom sadr\u017Eaju.",
"Keep Formatting": "Zadr\u017Ei oblikovanje",
"Remove Formatting": "Ukloni oblikovanje",
"Local Image Import": "Uvoz lokalne slike",
"Trigger paste again from the keyboard to paste content with images.": "Ponovno aktivirajte lijepljenje na tipkovnici kako biste zalijepili sadr\u017Eaj sa slikama.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Potreban je Adobe Flash za uvoz slika iz programskog paketa Microsoft Office. Instaliraj <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Pritisnite <span class=\"ephox-polish-help-kbd\">ESC</span> kako biste zanemarili lokalne slike i nastavili s ure\u0111ivanjem.",
"Please wait...": "Pri\u010Dekajte...",
"Your browser security settings may be preventing images from being imported.": "Postavke za\u0161tite preglednika mo\u017Eda spre\u010Davaju uvoz slika.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Postavke za\u0161tite preglednika mo\u017Eda spre\u010Davaju uvoz slika. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Vi\u0161e informacija o lijepljenju za Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari ne podr\u017Eava izravno lijepljenje slika. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Vi\u0161e informacija o lijepljenju slika za Safari</a>",
"The images service was not found: (": "Usluga slika nije prona\u0111ena: (",
"Image failed to upload: (": "Slika nije u\u010Ditana: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Lijepljenje lokalne slike isklju\u010Deno je. Lokalne su slike uklonjene iz zalijepljenog sadr\u017Eaja."
});