body .jodit-container {
	border: none;
	height: calc(100% - 47px) !important;
	word-wrap: break-word;
}

body .jodit-workplace {
	height: 100% !important;
}

body .jodit-toolbar__box {
	border-color: #e6e6e6;
	border-left: none;
	border-right: none;
}

.jodit-container:not(.jodit_inline) {
	border: none;
}

.jodit-container .jodit-workplace strong,
.jodit-container .jodit-workplace b {
    font-weight: bold;
}

.jodit-container .jodit-workplace em,
.jodit-container .jodit-workplace i {
    font-style: italic;
}

.jodit-container .jodit-workplace b em,
.jodit-container .jodit-workplace b i,
.jodit-container .jodit-workplace strong em,
.jodit-container .jodit-workplace strong i {
	font-weight: bold;
}

.jodit-container .jodit-workplace ul,
.jodit-container .jodit-workplace ol {
	padding-left: 1em;
	list-style: disc;
	list-style-position: outside;
}
.jodit-container .jodit-workplace ol {
	list-style: decimal;
}

.jodit-container .jodit-workplace li {
	padding: 2px 2px;
    margin-left: 1em;
}

.jodit-container .jodit-workplace blockquote {
	font-style: normal;
}
.jodit-container .jodit-workplace pre, .jodit-container .jodit-workplace div {
	white-space: normal !important;
}

.jodit-toolbar__box:not(:empty) {
	border-top: 1px solid #e6e6e6;
    border-bottom: 1px solid #e6e6e6;
	border-radius: unset;
}