tinymce.addI18n('tr', {
"Paste Formatting Options": "Bi\u00E7imlendirme Se\u00E7eneklerini Yap\u0131\u015Ft\u0131r",
"Choose to keep or remove formatting in the pasted content.": "Yap\u0131\u015Ft\u0131r\u0131lan i\u00E7erikteki bi\u00E7imlendirmeyi korumay\u0131 veya kald\u0131rmay\u0131 se\u00E7in.",
"Keep Formatting": "Bi\u00E7imlendirmeyi Koru",
"Remove Formatting": "Bi\u00E7imlendirmeyi Kald\u0131r",
"Local Image Import": "Yerel Resim Alma",
"Trigger paste again from the keyboard to paste content with images.": "Resimli i\u00E7eri\u011Fi yap\u0131\u015Ft\u0131rmak i\u00E7in klavyeden tekrar yap\u0131\u015Ft\u0131r i\u015Flemi yap\u0131n.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Resimleri Microsoft Office belgesinden almak i\u00E7in Adobe Flash gereklidir. Y\u00FCkle: <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Yerel resimleri yoksaymak ve d\u00FCzenlemeye devam etmek i\u00E7in <span class=\"ephox-polish-help-kbd\">ESC</span> tu\u015Funa bas\u0131n.",
"Please wait...": "L\u00FCtfen bekleyin...",
"Your browser security settings may be preventing images from being imported.": "Taray\u0131c\u0131 g\u00FCvenlik ayarlar\u0131n\u0131z resimlerin al\u0131nmas\u0131n\u0131 engelliyor olabilir.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Taray\u0131c\u0131 g\u00FCvenlik ayarlar\u0131n\u0131z resimlerin al\u0131nmas\u0131n\u0131 engelliyor olabilir. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Safari i\u00E7in yap\u0131\u015Ft\u0131rma ile ilgili daha fazla bilgi</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari resimlerin do\u011Frudan yap\u0131\u015Ft\u0131r\u0131lmas\u0131n\u0131 desteklemez. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Safari i\u00E7in resim yap\u0131\u015Ft\u0131rma ile ilgili daha fazla bilgi</a>",
"The images service was not found: (": "Resim hizmeti bulunamad\u0131: (",
"Image failed to upload: (": "Resim y\u00FCkleme ba\u015Far\u0131s\u0131z: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Yerel resim yap\u0131\u015Ft\u0131rma devre d\u0131\u015F\u0131. Yerel resimler yap\u0131\u015Ft\u0131r\u0131lan i\u00E7erikten kald\u0131r\u0131ld\u0131."
});