.zarafa-mailcreatepanel-field-editor,
.k-signature-editor  {
	/* Remove skin creator's dark grey border-color */
	.mce-btn.mce-active,.mce-btn.mce-active:hover {
		border-color: #ebebeb;
	}

	/* Remove skin creator's dark grey border-color */
	.mce-btn:hover,.mce-btn:focus {
		border-color: #ebebeb;
	}

	.mce-toolbar-grp {
		padding-bottom: 0px;
	}

	/* Override inline style of tinymce */
	.mce-panel.mce-tinymce {
		border-width: 1px 0px 0px !important;
		width: auto !important;
	}
}

.mce-arrow-up {
	margin-top: 12px !important;
}

.mce-arrow-down {
	margin-top: -12px !important;
}

.mce-arrow:before,
.mce-arrow:after {
	position: absolute;
	left: 50%;
	display: block;
	width: 0;
	height: 0;
	border-style: solid;
	border-color: transparent;
	content: "";
}

.mce-arrow.mce-arrow-up:before {
	top: -9px;
	border-bottom-color: $grey ;
	border-width: 0 9px 9px;
	margin-left: -9px;
}

.mce-arrow.mce-arrow-down:before {
	bottom: -9px;
	border-top-color: $grey ;
	border-width: 9px 9px 0;
	margin-left: -9px;
}

.mce-arrow.mce-arrow-up:after {
	top: -8px;
	border-bottom-color: $white;
	border-width: 0 8px 8px;
	margin-left: -8px;
}

.mce-arrow.mce-arrow-down:after {
	bottom: -8px;
	border-top-color: $white;
	border-width: 8px 8px 0;
	margin-left: -8px;
}
