tinymce.addI18n('nb_NO', {
"Paste Formatting Options": "Lim formateringsvalg",
"Choose to keep or remove formatting in the pasted content.": "Velges for \u00E5 beholde formateringsvalgene i det limte innholdet.",
"Keep Formatting": "Behold formatering",
"Remove Formatting": "Fjern formatering",
"Local Image Import": "Lokal bildeimport",
"Trigger paste again from the keyboard to paste content with images.": "Utl\u00F8s liming igjen fra tastaturet for \u00E5 lime inn innhold med bilder.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Det kreves Adobe Flash for \u00E5 importere bilder fra Microsoft Office. Installer <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Trykk <span class=\"ephox-polish-help-kbd\">ESC</span> for \u00E5 ignorere lokale bilder og fortsette redigering.",
"Please wait...": "Vent litt ...",
"Your browser security settings may be preventing images from being imported.": "Din nettlesers sikkerhetsinnstillinger kan hindre bildene fra \u00E5 bli importert.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Din nettlesers sikkerhetsinnstillinger kan hindre bildene fra \u00E5 bli importert. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Mer informasjon om liming for Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari st\u00F8tter ikke direkte liming av bilder. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Mer informasjon om bildeliming for Safari</a>",
"The images service was not found: (": "Bildetjenesten ble ikke funnet: (",
"Image failed to upload: (": "Bildet ble ikke opplastet: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Liming av lokal bilder er deaktivert. Lokale bilder er fjernet fra limt innhold."
});