/* Rules settings category */
#zarafa-mainpanel-contentpanel-settings .k-rules-widget {
	> .x-panel-bwrap > .x-panel-body {
		padding: $padding-medium 0;
	}

	.k-store-picker {
		padding: 0 $padding-medium $padding-medium;
	}
}

.k-rulesgrid {
	.x-panel-tbar .x-toolbar {
		padding: $padding-medium;
	}

	.x-btn-noicon .x-btn-small {
		margin-right: $padding-medium;
	}

	.x-panel-body {
		padding: $padding-medium*2 !important;
	}
}

/* Rules edit panel (called "Inbox Rule" */

.k-ruleseditpanel {
	.zarafa-folder-link, .zarafa-user-link, .zarafa-word-link {
        	text-decoration: underline;
	        padding: 4px 0px 0px 5px;
	        cursor:pointer;
	        overflow: hidden;
        	text-overflow: ellipsis;
	        white-space: nowrap;
	        font-weight: bold;
	}
	
	 .zarafa-compositefield {
		padding-top: $padding-medium;
	}	

}
