tinymce.addI18n('ko_KR', {
"Paste Formatting Options": "\uBD99\uC5EC\uB123\uAE30 \uC11C\uC2DD \uC635\uC158",
"Choose to keep or remove formatting in the pasted content.": "\uBD99\uC5EC \uB123\uC740 \uCF58\uD150\uCE20\uC758 \uC11C\uC2DD\uC744 \uC720\uC9C0 \uB610\uB294 \uC81C\uAC70\uD558\uAE30 \uC704\uD574 \uC120\uD0DD\uD569\uB2C8\uB2E4.",
"Keep Formatting": "\uC11C\uC2DD \uC720\uC9C0",
"Remove Formatting": "\uC11C\uC2DD \uC81C\uAC70",
"Local Image Import": "\uB85C\uCEEC \uC774\uBBF8\uC9C0 \uAC00\uC838\uC624\uAE30",
"Trigger paste again from the keyboard to paste content with images.": "\uC774\uBBF8\uC9C0\uAC00 \uC788\uB294 \uCF58\uD150\uCE20\uB97C \uBD99\uC5EC \uB123\uC73C\uB824\uBA74 \uD0A4\uBCF4\uB4DC\uC5D0\uC11C \uBD99\uC5EC \uB123\uAE30\uB97C \uB2E4\uC2DC \uC0AC\uC6A9\uD558\uC2ED\uC2DC\uC624.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Microsoft Office\uC5D0\uC11C \uC774\uBBF8\uC9C0\uB97C \uAC00\uC838\uC624\uB824\uBA74 Adobe Flash\uAC00 \uD544\uC694\uD569\uB2C8\uB2E4. \uB2E4\uC74C\uC744 \uC124\uCE58\uD558\uC2ED\uC2DC\uC624: <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "\uB85C\uCEEC \uC774\uBBF8\uC9C0\uB97C \uBB34\uC2DC\uD558\uACE0 \uACC4\uC18D \uD3B8\uC9D1\uD558\uB824\uBA74 <span class=\"ephox-polish-help-kbd\">ESC</span>\uC744 \uB204\uB974\uC2ED\uC2DC\uC624.",
"Please wait...": "\uAE30\uB2E4\uB824 \uC8FC\uC2ED\uC2DC\uC624...",
"Your browser security settings may be preventing images from being imported.": "\uBE0C\uB77C\uC6B0\uC800 \uBCF4\uC548 \uC124\uC815\uC774 \uC774\uBBF8\uC9C0 \uAC00\uC838\uC624\uAE30\uB97C \uD5C8\uB77D\uD558\uC9C0 \uC54A\uC744 \uC218 \uC788\uC2B5\uB2C8\uB2E4.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "\uBE0C\uB77C\uC6B0\uC800 \uBCF4\uC548 \uC124\uC815\uC774 \uC774\uBBF8\uC9C0 \uAC00\uC838\uC624\uAE30\uB97C \uD5C8\uB77D\uD558\uC9C0 \uC54A\uC744 \uC218 \uC788\uC2B5\uB2C8\uB2E4. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Safari\uC5D0\uC11C \uBD99\uC5EC\uB123\uAE30\uC5D0 \uAD00\uD55C \uC790\uC138\uD55C \uB0B4\uC6A9</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari\uC5D0\uC11C\uB294 \uC774\uBBF8\uC9C0 \uC9C1\uC811 \uBD99\uC5EC\uB123\uAE30\uB97C \uC9C0\uC6D0\uD558\uC9C0 \uC54A\uC2B5\uB2C8\uB2E4. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Safari\uC5D0\uC11C \uC774\uBBF8\uC9C0 \uBD99\uC5EC\uB123\uAE30\uC5D0 \uAD00\uD55C \uC790\uC138\uD55C \uB0B4\uC6A9</a>",
"The images service was not found: (": "\uC774\uBBF8\uC9C0 \uC11C\uBE44\uC2A4\uB97C \uCC3E\uC744 \uC218 \uC5C6\uC2B5\uB2C8\uB2E4: (",
"Image failed to upload: (": "\uC774\uBBF8\uC9C0\uAC00 \uC5C5\uB85C\uB4DC \uB418\uC9C0 \uC54A\uC558\uC2B5\uB2C8\uB2E4: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "\uB85C\uCEEC \uC774\uBBF8\uC9C0 \uBD99\uC5EC \uB123\uAE30 \uC0AC\uC6A9\uC774 \uC911\uC9C0\uB418\uC5C8\uC2B5\uB2C8\uB2E4. \uB85C\uCEEC \uC774\uBBF8\uC9C0\uAC00 \uBD99\uC5EC \uB123\uC740 \uB0B4\uC6A9\uC5D0\uC11C \uC0AD\uC81C \uB418\uC5C8\uC2B5\uB2C8\uB2E4."
});