tinymce.addI18n('zh_TW', {
"Paste Formatting Options": "\u7C98\u8CBC\u683C\u5F0F\u9078\u9805",
"Choose to keep or remove formatting in the pasted content.": "\u9078\u64C7\u5728\u7C98\u8CBC\u7684\u5167\u5BB9\u4FDD\u6301\u6216\u79FB\u9664\u683C\u5F0F\u3002",
"Keep Formatting": "\u4FDD\u6301\u683C\u5F0F",
"Remove Formatting": "\u79FB\u9664\u683C\u5F0F",
"Local Image Import": "\u5C0E\u5165\u672C\u5730\u5716\u50CF",
"Trigger paste again from the keyboard to paste content with images.": "\u518D\u6B21\u5F9E\u9375\u76E4\u89F8\u767C\u7C98\u8CBC\u64CD\u4F5C\uFF0C\u7C98\u8CBC\u542B\u5716\u50CF\u7684\u5167\u5BB9\u3002",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "\u9700\u8981Adobe Flash\u624D\u80FD\u5F9E\u5FAE\u8EDF\u8FA6\u516C\u8EDF\u4EF6\u5C0E\u5165\u5716\u50CF\u3002 \u5B89\u88DD<a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "\u6309<span class=\"ephox-polish-help-kbd\">ESC</span>\u5FFD\u7565\u672C\u5730\u5716\u50CF\u4E26\u7E7C\u7E8C\u7DE8\u8F2F",
"Please wait...": "\u8ACB\u7A0D\u7B49......",
"Your browser security settings may be preventing images from being imported.": "\u700F\u89BD\u5668\u7684\u5B89\u5168\u8A2D\u7F6E\u53EF\u80FD\u963B\u6B62\u4E86\u5716\u50CF\u5C0E\u5165\u3002",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "\u700F\u89BD\u5668\u7684\u5B89\u5168\u8A2D\u7F6E\u53EF\u80FD\u963B\u6B62\u4E86\u5716\u50CF\u5C0E\u5165\u3002 <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">\u66F4\u591A\u95DC\u65BCSafari\u7C98\u8CBC\u64CD\u4F5C\u7684\u8CC7\u8A0A</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari\u700F\u89BD\u5668\u4E0D\u652F\u6301\u76F4\u63A5\u7C98\u8CBC\u5716\u50CF\u3002 <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">\u66F4\u591A\u95DC\u65BCSafari\u5716\u50CF\u7C98\u8CBC\u7684\u8CC7\u8A0A</a>",
"The images service was not found: (": "\u627E\u4E0D\u5230\u5716\u50CF\u670D\u52D9: (",
"Image failed to upload: (": "\u5716\u50CF\u4E0A\u50B3\u5931\u6557: (",
").": ")\u3002",
"Local image paste has been disabled. Local images have been removed from pasted content.": "\u5C40\u90E8\u5716\u50CF\u7C98\u8CBC\u5DF2\u88AB\u7981\u7528\u3002 \u5C40\u90E8\u5716\u50CF\u5DF2\u5F9E\u7C98\u8CBC\u5167\u5BB9\u8655\u522A\u9664\u3002"
});