tinymce.addI18n('ja', {
"Paste Formatting Options": "\u66F8\u5F0F\u8CBC\u308A\u4ED8\u3051\u30AA\u30D7\u30B7\u30E7\u30F3",
"Choose to keep or remove formatting in the pasted content.": "\u8CBC\u308A\u4ED8\u3051\u308B\u30B3\u30F3\u30C6\u30F3\u30C4\u306E\u66F8\u5F0F\u3092\u4FDD\u5B58\u3059\u308B\u304B\u524A\u9664\u3059\u308B\u304B\u3092\u9078\u629E\u3057\u307E\u3059\u3002",
"Keep Formatting": "\u66F8\u5F0F\u3092\u4FDD\u5B58",
"Remove Formatting": "\u66F8\u5F0F\u3092\u524A\u9664",
"Local Image Import": "\u30ED\u30FC\u30AB\u30EB\u753B\u50CF\u30A4\u30F3\u30DD\u30FC\u30C8",
"Trigger paste again from the keyboard to paste content with images.": "\u30AD\u30FC\u30DC\u30FC\u30C9\u304B\u3089\u8CBC\u308A\u4ED8\u3051\u3092\u518D\u5B9F\u884C\u3057\u3001\u753B\u50CF\u304C\u542B\u307E\u308C\u308B\u30B3\u30F3\u30C6\u30F3\u30C4\u3092\u8CBC\u308A\u4ED8\u3051\u307E\u3059\u3002",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Microsoft Office\u304B\u3089\u753B\u50CF\u3092\u30A4\u30F3\u30DD\u30FC\u30C8\u3059\u308B\u306B\u306F\u3001Adobe Flash Player\u304C\u5FC5\u8981\u3067\u3059\u3002 <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>\u3092\u30A4\u30F3\u30B9\u30C8\u30FC\u30EB\u3057\u3066\u304F\u3060\u3055\u3044\u3002",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "\u30ED\u30FC\u30AB\u30EB\u753B\u50CF\u3092\u7121\u8996\u3057\u3066\u7DE8\u96C6\u3092\u7D9A\u884C\u3059\u308B\u306B\u306F<span class=\"ephox-polish-help-kbd\">ESC\u30AD\u30FC</span>\u3092\u62BC\u3057\u307E\u3059\u3002",
"Please wait...": "\u304A\u5F85\u3061\u304F\u3060\u3055\u3044...",
"Your browser security settings may be preventing images from being imported.": "\u304A\u4F7F\u3044\u306E\u30D6\u30E9\u30A6\u30B6\u306E\u30BB\u30AD\u30E5\u30EA\u30C6\u30A3\u8A2D\u5B9A\u306E\u305F\u3081\u306B\u753B\u50CF\u304C\u30A4\u30F3\u30DD\u30FC\u30C8\u3067\u304D\u3066\u3044\u306A\u3044\u53EF\u80FD\u6027\u304C\u3042\u308A\u307E\u3059\u3002",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "\u304A\u4F7F\u3044\u306E\u30D6\u30E9\u30A6\u30B6\u306E\u30BB\u30AD\u30E5\u30EA\u30C6\u30A3\u8A2D\u5B9A\u306E\u305F\u3081\u306B\u753B\u50CF\u304C\u30A4\u30F3\u30DD\u30FC\u30C8\u3067\u304D\u3066\u3044\u306A\u3044\u53EF\u80FD\u6027\u304C\u3042\u308A\u307E\u3059\u3002 <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Safari\u3067\u306E\u8CBC\u308A\u4ED8\u3051\u306B\u3064\u3044\u3066\u306E\u8A73\u7D30</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari\u306F\u753B\u50CF\u306E\u76F4\u63A5\u8CBC\u308A\u4ED8\u3051\u3092\u30B5\u30DD\u30FC\u30C8\u3057\u3066\u3044\u307E\u305B\u3093\u3002 <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Safari\u3067\u306E\u753B\u50CF\u8CBC\u308A\u4ED8\u3051\u306B\u3064\u3044\u3066\u306E\u8A73\u7D30</a>",
"The images service was not found: (": "\u753B\u50CF\u30B5\u30FC\u30D3\u30B9\u304C\u898B\u3064\u304B\u308A\u307E\u305B\u3093\u3067\u3057\u305F\uFF1A (",
"Image failed to upload: (": "\u753B\u50CF\u306F\u30A2\u30C3\u30D7\u30ED\u30FC\u30C9\u3067\u304D\u307E\u305B\u3093\u3067\u3057\u305F\uFF1A (",
").": ")\u3002",
"Local image paste has been disabled. Local images have been removed from pasted content.": "\u30ED\u30FC\u30AB\u30EB\u753B\u50CF\u306E\u8CBC\u308A\u4ED8\u3051\u306F\u7121\u52B9\u5316\u3055\u308C\u3066\u3044\u307E\u3059\u3002 \u30ED\u30FC\u30AB\u30EB\u753B\u50CF\u304C\u8CBC\u308A\u4ED8\u3051\u3089\u308C\u305F\u30B3\u30F3\u30C6\u30F3\u30C4\u304B\u3089\u524A\u9664\u3055\u308C\u307E\u3057\u305F\u3002"
});