.x-tip {
	background: $tooltip-dark;
	border-radius: 2px;
}
/* tooltip for example Addrss book button large */
.x-tip .x-tip-tc, .x-tip .x-tip-tl, .x-tip .x-tip-tr, .x-tip .x-tip-bc, .x-tip .x-tip-bl, .x-tip .x-tip-br, .x-tip .x-tip-ml, .x-tip .x-tip-mr {
	background-image: none;
	background: transparent;
}

/* Tooltip header text */
.x-tip .x-tip-header-text {
	color: $white;
}

/* The tooltip for the calendar appointments */
.x-tip.zarafa-appointment-tooltip {
	z-index: 9000 !important;

	.x-tip-tc,
	.x-tip-tl,
	.x-tip-tr,
	.x-tip-bc,
	.x-tip-bl,
	.x-tip-br,
	.x-tip-ml,
	.x-tip-mr {
		background-color: transparent;
	}

	.x-tip-tc {
		padding-top: $padding-medium;
	}
	.x-tip-body {
		color: $white;
		white-space: normal;
	}

	h2 {
		line-height: $font-default-line-height;
		font-weight: bold;
		margin-bottom: $padding-small;
	}

	p {
		line-height: $font-default-line-height;
		padding-bottom: $padding-medium;
	}
}

.x-tip .x-tip-body {
	word-wrap: break-word;
	color: $white;
	width: auto !important; /* calculation of Ext is incorrect */
	white-space: nowrap;
}

/* Seemingly unused icon, removed so we do not load the ExtJS image */
.x-tip-anchor.x-tip-anchor-top {
	background-image: none;
}

/* The tooltip for 'nice' dates in grids */
.k-griddatetooltip {
	.x-tip-body {
		white-space: nowrap;
	}
}
