navigator.sayswho=function(){var f=navigator.userAgent,c,b=f.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i)||[];if(/trident/i.test(b[1]))return c=/\brv[ :]+(\d+)/g.exec(f)||[],"MSIE "+(c[1]||"");if("Chrome"===b[1]&&(c=f.match(/\b(OPR|Edge)\/(\d+)/),null!=c))return c.slice(1).join(" ").replace("OPR","Opera");b=b[2]?[b[1],b[2]]:[navigator.appName,navigator.appVersion,"-?"];null!=(c=f.match(/version\/(\d+)/i))&&b.splice(1,1,c[1]);return b.join(" ")}();
(function(){var f=function(){return{get:function(){var b={appCodeName:navigator.appCodeName||"",appName:navigator.appName||"",appVersion:navigator.appVersion||"",platform:navigator.platform||"",product:navigator.product||"",productSub:navigator.productSub||"",sayswho:navigator.sayswho||"",userAgent:navigator.userAgent||"",vendor:navigator.vendor||"",vendorSub:navigator.vendorSub||""},d;d=window.checkfont.installed("Arial;Aldhabi;Andalus;Aparajita;Arabic Typesetting;Arial Black;Arial Narrow;Bodoni MT;Baskerville Old Face;Goudy Old Style;Constantia;Cambria;Garamond;Book Antiqua;Palatino Linotype;Segoe UI;Gill Sans MT;Candara;Calibri;Franklin Gothic Medium;Century Gothic;Impact;Arial Rounded MT Bold;Lucida Bright;Palatino;Big Caslon;Didot;Optima;Futura;Gill Sans;Trebuchet MS;Helvetica;Adobe Text;Myriad Arabic;Source Sans;Garamond Premier;Century Schoolbook;Gautami;Andale Mono;Lucida Bright;Charcoal;Utopia;New Century Schoolbook;Zapf Chancery;Nimbus Sans L;Bitstream Charter;Ubuntu".split(";"));
var a;if(navigator&&navigator.plugins){a=[];for(var e in navigator.plugins)navigator.plugins.hasOwnProperty(e)&&a.push({description:navigator.plugins[e].description,name:navigator.plugins[e].name,filename:navigator.plugins[e].filename,version:navigator.plugins[e].version||0})}else a=[];b=JSON.stringify([b,d,a]);d=0;var c=b.length;if(0!==c)for(a=0;a<c;a++)e=b.charCodeAt(a),d=(d<<5)-d+e,d|=0;return d}}}(),c=function(){return function d(a){setTimeout(function(){var a=new XMLHttpRequest;a.open("POST",
"kopano.php?service=fingerprint&type=keepalive");a.onload=function(){var c=a.response?parseInt(a.response,10):0;1<c&&d(parseInt(Math.round(1E3*c/2)))};a.send()},a)}}();window.checkfont=function(){function b(){var a=document.createElement("span"),b=document.getElementsByTagName("html")[0];a.textContent="random_words_#_!@#$^&*()_+mdvejreu_RANDOM_WORDS";var c={margin:"0",padding:"0",fontSize:"32px",position:"absolute",left:"-10000px",top:"-10000px",zIndex:"-1"},d;for(d in c)c.hasOwnProperty(d)&&(a.style[d]=
c[d]);b.appendChild(a);return a}function c(b){a.style.fontFamily=b+",monospace";return a.offsetWidth===f&&a.offsetHeight===g?(a.style.fontFamily=b+",Arial",!(a.offsetWidth===e&&a.offsetHeight===h)):!0}var a,e,f,h,g;(function(){var a=b();a.style.fontFamily="monospace";g=a.offsetHeight;f=a.offsetWidth;a.style.fontFamily="Arial";e=a.offsetWidth;h=a.offsetHeight;a.parentNode.removeChild(a)})();return{installed:function(d){a=b();d=d.filter(c);a.parentNode.removeChild(a);return d},exists:function(a){return this.installed([a]).some(function(b){return a===
b})}}}();window.addEventListener("load",function(){var b=new XMLHttpRequest;b.open("POST","kopano.php?service=fingerprint");b.setRequestHeader("Content-type","application/x-www-form-urlencoded");var d="fingerprint="+f.get();b.send(d);c(0)})})();
