tinymce.addI18n('fr_FR', {
"Paste Formatting Options": "Options de mise en forme du collage",
"Choose to keep or remove formatting in the pasted content.": "Choisir de conserver ou de supprimer la mise en forme du contenu coll\u00E9.",
"Keep Formatting": "Conserver la mise en forme",
"Remove Formatting": "Supprimer la mise en forme",
"Local Image Import": "Importation d\'une image locale",
"Trigger paste again from the keyboard to paste content with images.": "Pour coller un contenu avec des images, utilisez une nouvelle fois le raccourci clavier de collage.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash est n\u00E9cessaire \u00E0 l\'importation d\'images depuis Microsoft Office. Installez <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Appuyez sur la touche <span class=\"ephox-polish-help-kbd\">\u00C9chap</span> pour ignorer les images locales et continuer la modification.",
"Please wait...": "Merci de patienter...",
"Your browser security settings may be preventing images from being imported.": "Il est possible que les param\u00E8tres de s\u00E9curit\u00E9 de votre navigateur emp\u00EAchent l\'importation d\'images.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Il est possible que les param\u00E8tres de s\u00E9curit\u00E9 de votre navigateur emp\u00EAchent l\'importation d\'images. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Plus d\'informations sur le collage dans Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari ne prend pas en charge le collage direct d\'images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Plus d\'informations sur le collage d\'images dans Safari</a>",
"The images service was not found: (": "Le service d\'image est introuvable\u00A0: (",
"Image failed to upload: (": "\u00C9chec du chargement de l\'image\u00A0: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Le collage de l\'image locale a \u00E9t\u00E9 d\u00E9sactiv\u00E9. Les images locales ont \u00E9t\u00E9 supprim\u00E9es du contenu coll\u00E9."
});