<?php

namespace Sabre\HTTP;

/**
 * This class contains the version number for the HTTP package
 *
 * @copyright Copyright (C) fruux GmbH (https://fruux.com/)
 * @author Evert Pot (http://evertpot.com/)
 * @license http://sabre.io/license/ Modified BSD License
 */
class Version {

    /**
     * Full version number
     */
    const VERSION = '4.2.4';

}
