tinymce.addI18n('ro', {
"Paste Formatting Options": "Op\u0163iuni de formatare con\u0163inut lipit",
"Choose to keep or remove formatting in the pasted content.": "Alege\u0163i p\u0103strarea sau eliminarea format\u0103rii pentru con\u0163inut lipit.",
"Keep Formatting": "P\u0103strare formatare",
"Remove Formatting": "Eliminare formatare",
"Local Image Import": "Import imagine local\u0103",
"Trigger paste again from the keyboard to paste content with images.": "Declan\u015Fa\u0163i lipirea din nou de la tastatur\u0103 pentru a lipi con\u0163inutul cu imaginile.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash este necesar pentru importul imaginilor din Microsoft Office. Instala\u0163i <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Ap\u0103sa\u0163i <span class=\"ephox-polish-help-kbd\">ESC</span> pentru a ignora imaginile locale \u015Fi a continua editarea.",
"Please wait...": "V\u0103 rug\u0103m a\u015Ftepta\u0163i...",
"Your browser security settings may be preventing images from being imported.": "Set\u0103rile de securitate ale browser-ului dumneavoastr\u0103 pot \u00EEmpiedica importarea imaginilor.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Set\u0103rile de securitate ale browser-ului dumneavoastr\u0103 pot \u00EEmpiedica importarea imaginilor. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Mai multe informa\u0163ii privind lipirea pentru Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari nu accept\u0103 lipirea direct\u0103 a imaginilor. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Mai multe informa\u0163ii privind lipirea imaginilor pentru Safari</a>",
"The images service was not found: (": "Serviciul de imagini nu a fost g\u0103sit: (",
"Image failed to upload: (": "\u00CEnc\u0103rcarea imaginii a e\u015Fuat: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Lipirea imaginii locale a fost dezactivat\u0103. Imaginile locale au fost eliminate din con\u0163inutul lipit."
});