/* Style Add Widget Panel */

.k-widget-item {
	width:80px;
	text-align:center;
	display: block;
	float:left;
	margin:5px;
	cursor:pointer;
}
.k-widget-item .thumb-wrap {
	display: block;
	width:80px;
	height:80px;
	text-align:center;
	vertical-align:middle;
	position:relative;
}

.k-widget-item > span {
	height:2.75em;
	display:block;
	overflow:hidden;
	padding-top:0.25em;
}

.k-widgets-selectedwidget
{
	color : #7f7f7f;
}
.k-widgets-hoverwidget
{
	color : #7f7f7f;
	text-decoration : underline;
}

.k-widget-item .thumb-wrap img {
	display:block;
	margin:auto;
}

/* Body element of a panel */
.x-panel-btns {
	padding:5px;
}
.x-panel-tl, .x-panel-tc, .x-panel-tr {
	background-image:none;
	background-color:transparent;
	padding: 0;
}
.x-panel-tl {
	border: none;
}
.x-panel-ml, .x-panel-mc, .x-panel-mr {
	background-image:none;
	background-color:#EBEAEB;
}
.x-panel-bl, .x-panel-bc, .x-panel-br {
	background-image:none;
}
.x-panel.x-portlet {
	border: 1px solid $grey;
	margin-bottom: 10px;
	background-color: transparent !important;

	.x-panel-ml,
	.x-panel-mr {
		padding: 0;
	}

	.x-panel-bl.x-panel-nofooter {
		display: none;
	}
}

/* TaskWidget */
.k-taskwidget,
.k-appointmentwidget {
		.x-grid3-row {
		.x-grid3-row-body-tr {
			.zarafa-grid-body-container {
				.grid_compact_left {
					padding-left: $padding-medium + 1px;
				}
			}
		}
	}

	.x-grid3-row.zarafa-task-overdue {
		.x-grid3-row-body-tr {
			.zarafa-grid-body-container {
				.grid_compact_left {
					color: inherit;
					opacity: 0.5;
				}
			}
		}
	}
}

/*for Rules widget and Out of office settings widget*/
#zarafa-mainpanel-contentpanel-settings {
	.k-store-picker {
		padding: 0 0 10px;
		border-bottom: 1px solid $grey;
		margin-bottom: 10px;

        .x-form-item {
			margin-bottom: 0;
		}

		label {
			width: auto;
			border: 1px solid transparent; /* to align the text with the text in the combo */
		}
	}
}

#datepicker.k-widget-item, #tasks.k-widget-item,
#mail.k-widget-item, #appointments.k-widget-item,
#quickappt.k-widget-item {
	background-size: 70px;
	background-repeat: no-repeat !important;
	background-position: 7px 10px !important;
}

#quicktask.k-widget-item, #quicknote.k-widget-item  {
	background-size: 70px;
	background-repeat: no-repeat !important;
	background-position: 6px 10px !important;
}

#quickcontact.k-widget-item {
	background-size: 70px;
	background-repeat: no-repeat !important;
	background-position: 10px 10px !important;
}

#quickmail.k-widget-item {
	background-size: 70px;
	background-repeat: no-repeat !important;
	background-position: 6px 15px !important;
}

.k-configure-widget .x-form-cb-label {
	padding: 3px 3px 3px 4px;
	margin-left: 16px !important;
}

.k-folder-link {
	text-decoration: underline;
	padding: 4px 0px 0px 5px;
	cursor:pointer;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
	font-weight: bold;
}

/* do not load non visible gif's from ExtJS */
.x-resizable-over .x-resizable-handle-east, .x-resizable-pinned .x-resizable-handle-east, .x-resizable-over .x-resizable-handle-west, .x-resizable-pinned .x-resizable-handle-west {
	background-image: none !important;
}

.x-resizable-over .x-resizable-handle-south, .x-resizable-pinned .x-resizable-handle-south, .x-resizable-over .x-resizable-handle-north, .x-resizable-pinned .x-resizable-handle-north {
	background-image: none !important;
}

.x-resizable-over .x-resizable-handle-northeast, .x-resizable-pinned .x-resizable-handle-northeast {
	background-image: none !important;
}

.x-resizable-over .x-resizable-handle-northwest, .x-resizable-pinned .x-resizable-handle-northwest {
	background-image: none !important;
}

.x-resizable-over .x-resizable-handle-southeast, .x-resizable-pinned .x-resizable-handle-southeast {
	background-image: none !important;
}

.x-resizable-over .x-resizable-handle-southwest, .x-resizable-pinned .x-resizable-handle-southwest {
	background-image: none !important;
}
