#zarafa-mainpanel-contentpanel-settings {

	/* The categories panel */
	.zarafa-settings-category-panel {
		border-right: 1px solid $grey;
		padding-top: $padding-medium;
		z-index: 1;
		overflow: hidden !important;

		&:hover {
			overflow : auto !important;
		}

		.zarafa-settings-category-tab {
			padding: ($settings-category-tab-height - $font-default-line-height) $padding-medium;
			padding-left: 4px;
			line-height: $font-default-line-height;
			cursor: pointer;
			background-position: ($settings-category-tab-icon-strip-width - 16px)/2 center !important;
		}
		.zarafa-settings-category-tab.k-with-icon {
			img.k-settings-category-icon {
				width: 20px;
				height: 20px;
				float: left;
				background-position: center center !important;
				background-repeat: no-repeat;
				margin-right: $padding-medium - 1px;
			}
		}
		.zarafa-settings-category-tab-over {
			background-color: $grey2;
		}
		.zarafa-settings-category-tab-active {
			background-color: $light-blue;
		}
	}

	/* The main settings panel that contains all settings widgets */
	.zarafa-settings-category-widget-panel {
		padding: $padding-medium 0 0 0;

		.x-panel .x-panel-header, .x-panel-ghost .x-panel-header {
			background: transparent;
			font-size: $font-size-large;
			font-weight: bold;
		}

		.zarafa-settings-widget {
			margin-bottom: $padding-medium;

			.x-panel-header {
				padding: ($settings-category-tab-height - $font-default-line-height)/2 $padding-medium*2;
				line-height: $font-default-line-height;
				border: none;
			}

			.x-panel-body {
				padding: $padding-medium*2;
				border-left: none;
				border-right: none;
			}

			.x-panel-body .x-panel .x-panel-header {
				font-weight: normal;
				padding: 15px 0 15px 0;
			}

			.x-panel-body .x-panel .x-panel-body {
				border-bottom: none;
				padding: 0;
			}

			.x-panel-body .x-panel .x-grid3-header {
				border-bottom: none;
			}

			.x-panel-body .x-panel .x-grid3-scroller {
				border: 1px solid $grey;
			}

			.zarafa-quotabar {
				border: 1px solid $grey2;
				overflow: hidden;
				height: $button-small-height - 2px; /* subtract 2 for the border */

				.zarafa-quotabar-normal {
					display: inline-block;
					background: $light-blue;
					height: $button-small-height - 2px; /* subtract 2 for the border */
				}
				.zarafa-quotabar-warn {
					display: inline-block;
					background: #EEE92B;
					height: $button-small-height - 2px; /* subtract 2 for the border */
				}
				.zarafa-quotabar-soft {
					display: inline-block;
					background: #FFAA66;
					height: $button-small-height - 2px; /* subtract 2 for the border */
				}
				.zarafa-quotabar-hard {
					display: inline-block;
					background: #FF6666;
					height: $button-small-height - 2px; /* subtract 2 for the border */
				}
			}

			.zarafa-quota-string {
				font-weight: bold;
			}

		}

		.zarafa-settings-widget:last-child {
			margin-bottom: 0;
		}

		/* minor information in labels */
		.k-settings-label-minor {
			opacity: 0.5;
			margin-left: $padding-small;
		}

		/* Keyboard shortcuts */
		.zarafa-settings-keyboard-shortcut-row {
			background-color: $grey3;
			padding: $padding-small;
			border: 1px solid $grey;
		}

		/* About Category */
		.zarafa-settings-about-text {
			p {
				padding-bottom: 1em;
			}

			h1 {
				margin-bottom: 0.5em;
				margin-top: 0.5em;
				font-size: 1.2em !important;
			}

			a {
				background: $kopano-blue;
				color: $white;
				padding: $padding-small;
				text-decoration: none;
			}
			a:hover {
				background: $kopano-blue-hover;
			}
		}

		/* Advanced category */
		.k-settings-category-advanced-container {
			.k-settings-search-field {
				height: 30px !important;
				font-size: medium;
			}
		}

	}
}

/* Widget info text */
.zarafa-settings-widget-extrainfo {
	padding: 0 0 $padding-large 0;
}

/* Loading of "address book -> select default folder" */
.x-mask-loading-combo, .x-mask-loading-combo div {
	background-color: transparent !important;
	border: none !important;
}

/* Settings icons in the settingstab */
.zarafa-settings-category-tab, .zarafa-settings-favorite-tab {
	background-position: 2px center;
	background-repeat: no-repeat;
}

.zarafa-settings-signatures {
	margin-right: $padding-large;

	.x-panel-body-noheader {
		border-top: none;
	}
}

.k-signature-name, .zarafa-settings-oof-subject input {
	padding: $padding-medium $padding-medium $padding-medium 2*$padding-medium;
	font-size: $font-size-large !important;
}

.k-signature-editor textarea {
	padding: $padding-extra-large;
	font-family: monospace !important;
}

.k-signature-editor {
	.mce-panel.mce-tinymce {
		width: auto !important;
	}
}

.keyboard-type {
	margin-left: 40px;
	padding-left: $padding-medium;
	padding-right: $padding-medium;
	line-height: $font-line-height-large;
	border: solid 1px $grey;
	border-radius: 2px;
	background-color: $white;
	display: inline-block;
}

/* Out of office Category */
.zarafa-settings-oof-invalid {
	color: $red;
}

.zarafa-settings-oof-table-cellalign {
	text-align: right;
}

.zarafa-settings-oof-table-cellpadding {
	padding: $padding-medium $padding-medium $padding-medium 0;
}

.zarafa-settings-oof-subject .label-subject {
	line-height: $button-small-height;
	padding-right: $padding-medium;
}

.zarafa-settings-oof-autoreplay {
	padding: $padding-large 0 $padding-medium 0;
}

/* Plugin list */
.k-settings-pluginpanel {

	.x-panel-body {
		padding: 0;
		border: none;
	}

	.x-panel-header {
		border: none;
	}

	.zarafa-settings-pluginavailable {

		.x-panel-body {
			border-top: none;
			padding: 0 !important;
		}

		.k-settings-plugingrid .x-grid3-scroller {
			border: none !important;
		}

		.k-settings-plugingrid .x-grid3-row .x-grid3-check-col-td, .x-grid3-row .x-grid3-td-checker {
			padding-left: 10px;
		}

		.zarafa-settings-pluginavailable-fixed {
			.x-grid3-col-display_name {
				span {
					font-style: italic;
					color: $grey6;
				}
			}
		}
	}
}
