.@{prefix}-container-body .@{prefix}-resizehandle {
	position: absolute;
	right: 0;
	bottom: 0;
	width: 16px;
	height: 16px;
	visibility: visible;
	cursor: s-resize;
	margin: 0;
}

.@{prefix}-container-body .@{prefix}-resizehandle-both {
	cursor: se-resize;
}

i.@{prefix}-i-resize {
	color: @text;
}
