tinymce.addI18n('sv_SE', {
"Paste Formatting Options": "Alternativa inklistringsformateringar",
"Choose to keep or remove formatting in the pasted content.": "V\u00E4lj att beh\u00E5lla eller ta bort formatering i det inklistrade inneh\u00E5llet.",
"Keep Formatting": "Beh\u00E5ll formatering",
"Remove Formatting": "Ta bort formatering",
"Local Image Import": "Importera lokal bild",
"Trigger paste again from the keyboard to paste content with images.": "Aktivera inklistring igen fr\u00E5n tangentbordet f\u00F6r att klistra in inneh\u00E5ll med bilder.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash kr\u00E4vs f\u00F6r att importera bilder fr\u00E5n Microsoft Office. Installera <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Tryck p\u00E5 <span class=\"ephox-polish-help-kbd\">ESC</span> f\u00F6r att ignorera lokala bilder och forts\u00E4tta redigera.",
"Please wait...": "V\u00E4nta...",
"Your browser security settings may be preventing images from being imported.": "Din webbl\u00E4sares s\u00E4kerhetsinst\u00E4llningar kan eventuellt f\u00F6rhindra import av bilder.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Din webbl\u00E4sares s\u00E4kerhetsinst\u00E4llningar kan eventuellt f\u00F6rhindra import av bilder. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Mer information om inklistring f\u00F6r Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari har inte st\u00F6d f\u00F6r direkt inklistring av bilder. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Mer information om inklistring av bilder f\u00F6r Safari</a>",
"The images service was not found: (": "Hittade inte bildtj\u00E4nsten: (",
"Image failed to upload: (": "Bilden kunde inte laddas upp: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Inklistring av lokala bilder har inaktiverats. Lokala bilder har tagits bort fr\u00E5n inklistrat inneh\u00E5ll."
});