tinymce.addI18n('sl_SI', {
"Paste Formatting Options": "Prilepi mo\u017Enosti oblikovanja",
"Choose to keep or remove formatting in the pasted content.": "Izberi ohranitev ali odstranitev oblikovanja v prilepljeni vsebini.",
"Keep Formatting": "Nadaljuj z oblikovanjem",
"Remove Formatting": "Odstrani oblikovanje",
"Local Image Import": "Uvoz lokalne slike",
"Trigger paste again from the keyboard to paste content with images.": "Znova spro\u017Eite lepljenje iz tipkovnice, da prilepite vsebino s slikami.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Za uvoz slik iz programa Microsoft Office je potreben program Adobe Flash. Namestite <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Pritisnite <span class=\"ephox-polish-help-kbd\">ESC</span>, da prezrete lokalne slike in nadaljujete z urejanjem.",
"Please wait...": "Po\u010Dakajte ...",
"Your browser security settings may be preventing images from being imported.": "Varnostne nastavitve va\u0161ega brskalnika morda prepre\u010Dujejo, da bi bile slike name\u0161\u010Dene.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Varnostne nastavitve va\u0161ega brskalnika morda prepre\u010Dujejo, da bi bile slike name\u0161\u010Dene. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Ve\u010D informacij o lepljenju za Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari ne podpira neposrednega lepljenja slik. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Ve\u010D informacij o lepljenju slik za Safari</a>",
"The images service was not found: (": "Storitve za slike ni bilo mogo\u010De najti: (",
"Image failed to upload: (": "Slike ni bilo mogo\u010De nalo\u017Eiti: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Funkcija lepljenja lokalne slike je bila onemogo\u010Dena. Lokalne slike so bile odstranjene iz lepljene vsebine."
});