// ImagePanel

.@{prefix}-imagepanel {
	overflow: auto;
	background: black;
}

.@{prefix}-imagepanel img {
	position: absolute;
}

.@{prefix}-imagetool.@{prefix}-btn .@{prefix}-ico {
	display: block;
	width: 20px;
	height: 20px;
	text-align: center;
	line-height: 20px;
	font-size: 20px;
	padding: 5px;
}
