tinymce.addI18n('da', {
"Paste Formatting Options": "Inds\u00E6t formateringsmuligheder",
"Choose to keep or remove formatting in the pasted content.": "V\u00E6lg at beholde eller fjerne formattering i det indsatte indhold.",
"Keep Formatting": "Behold formatering",
"Remove Formatting": "Fjern formatering",
"Local Image Import": "Import af lokalt billede",
"Trigger paste again from the keyboard to paste content with images.": "Udl\u00F8s inds\u00E6t fra tastaturet igen for at inds\u00E6tte indhold med billeder.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Der kr\u00E6ves Adobe Flash for at importere billeder fra Microsoft Office. Installer <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Tryk p\u00E5 <span class=\"ephox-polish-help-kbd\">ESC</span> for at ignorere lokale billeder og forts\u00E6tte med at redigere.",
"Please wait...": "Vent venligst...",
"Your browser security settings may be preventing images from being imported.": "Din browsers sikkerhedsindstillinger forhindrer muligvis, at billeder bliver importeret.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Din browsers sikkerhedsindstillinger forhindrer muligvis, at billeder bliver importeret. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Mere information om inds\u00E6ttelse til Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari underst\u00F8tter ikke direkte inds\u00E6ttelse af billeder. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Mere information om inds\u00E6ttelse af billeder til Safari</a>",
"The images service was not found: (": "Billedtjenesten blev ikke fundet: (",
"Image failed to upload: (": "Billedet blev ikke oploadet: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Lokal billedinds\u00E6tning er blevet sl\u00E5et fra. Lokale billeder er blevet fjernet fra indsat indhold."
});