tinymce.addI18n('it', {
"Paste Formatting Options": "Opzioni incolla formattazione",
"Choose to keep or remove formatting in the pasted content.": "Scegli se mantenere o rimuovere la formattazione nel contenuto incollato.",
"Keep Formatting": "Mantieni formattazione",
"Remove Formatting": "Rimuovi formattazione",
"Local Image Import": "Importa immagine locale",
"Trigger paste again from the keyboard to paste content with images.": "Incollare di nuovo usando la scorciatoia da tastiera per incollare il contenuto insieme alle immagini.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "E\u2019 necessario disporre di Adobe Flash per importare le immagini da Microsoft Office. Installa <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Premi <span class=\"ephox-polish-help-kbd\">ESC</span> per ignorare le immagini locali e procedere alle modifiche.",
"Please wait...": "Attendi prego...",
"Your browser security settings may be preventing images from being imported.": "Le impostazioni di sicurezza del browser potrebbero impedire l\u2019importazione delle immagini.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Le impostazioni di sicurezza del browser potrebbero impedire l\u2019importazione delle immagini. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Per maggiori informazioni su come incollare informazioni in Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari non consente di incollare direttamente le immagini. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Per maggiori informazioni su come incollare le immagini in Safari</a>",
"The images service was not found: (": "Il servizio immagini non \u00E8 stato trovato: (",
"Image failed to upload: (": "Impossibile caricare l\u2019immagine: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "L\u2019opzione \u2018incolla direttamente le immagini\u2019 \u00E8 stata disabilitata. Le immagini locali sono state rimosse dal contenuto incollato."
});