<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Icewind\\Streams\\Tests\\' => array($vendorDir . '/icewind/streams/tests'),
    'Icewind\\Streams\\' => array($baseDir . '/src', $vendorDir . '/icewind/streams/src'),
    'Icewind\\SMB\\' => array($baseDir . '/src'),
);
