#k-listview{font-family:arial, tahoma, helvetica, sans-serif}#k-name{font-size:16px;padding:15px 0 15px 15px}#k-printlist{width:100%;border-collapse:collapse}#k-printlist th{border:1px solid #ddd;padding:8px;font-size:13px;background-color:#f5f5f5;text-align:left;color:#222}#k-printlist td{padding:8px;font-size:11px;color:#222;border:1px solid #ddd}#k-printlist tr:nth-child(odd){background-color:#f5f5f5}#k-printlistbottom{width:100%;padding-top:5px;color:#222;font-size:12px}
/*# sourceMappingURL=print.list.css.map */
