/*
 * Contains all the colors that are used in the new design.
 * This file should only contain sass definitions, no actual css rules
 */

/* First we define some colors */
$kopano-blue              : #0f70bd;
$kopano-blue-hover        : #065fa5;
$kopano-blue-active       : #065fa5;

$grey                     : #e6e6e6;
$grey2                    : #dadada;
$grey3                    : #f8f8f8;
$grey4                    : #f2f2f2;
$grey5                    : #b6b6b6;
$grey6                    : #777777;
$grey7                    : #4c4c4c;
$lightgrey                : lightgrey;

$dark-grey                : #333333;
$dark-grey2               : #222222;

$green                    : #5ebb4b;
$green-hover              : #59b147;

$green-bright             : #23cb02;

$white                    : #ffffff;
$whiteSmoke               : whiteSmoke;
$black                    : #000000;
$red                      : #FF0000;
$red1                     : #e30022;
$red2                     : #fbdde1;
$sky-blue                 : #3094e3;

$light-blue               : #d5e8f7;
$seperator-color          : #aca889;

$yellow-light             : #fff3cd;
$yellow-dark              : #856404;

/* Now we define which color we use for different elements */

$background-color-main    : $white;

$background-color-header  : $grey;
$background-color-button  : $grey;

/* toolbars */
$background-color-toolbar : $white;
$seperator-color-toolbar  : $seperator-color;

/* tabs */
$background-color-tab                 : $grey;
$background-color-tab-active          : $white;
$background-color-tab-hover           : $grey;

/* menus */
$background-color-menu    : $white;
$border-color-menu        : $grey5;

/* grids */
$grid-hover-color         : $grey4;
$grid-active-color        : $light-blue;
$grid-subject-color       : $grey6;
$grid-info-font-color     : $yellow-dark;
$grid-info-bg-color       : $yellow-light;

/* fonts */
$font-color-light         : $white;
$font-color-dark          : $black;

/* Tooltip */
$tooltip-dark             : rgba(0, 0, 0, 0.8);

$preview-panel-timestamp     : $grey7;
