/* SMB BACKEND STYLES */

/* The main icon for the backend*/
/* Naming convention: icon_<size>_<backend name> */
.icon_16_SMB {
    background-image: url(../icons/smb16.png)!important;
    background-repeat: no-repeat;
}

.icon_24_SMB {
    background-image: url(../icons/smb24.png)!important;
    background-repeat: no-repeat;
}

.icon_256_SMB {
    background-image: url(../icons/smb256.png)!important;
    background-repeat: no-repeat;
}

.icon_16_logo_SMB {
    background-image: url(../icons/smb_logo16.png)!important;
    background-repeat: no-repeat;
}