/**********************************************************************
 * Mail create panel
 *********************************************************************/

.zarafa-mailcreatepanel {
	> .x-panel-bwrap {
		> .x-panel-body {
			padding: $padding-medium 0;

			.zarafa-mailcreatepanel-field-from,
			.zarafa-mailcreatepanel-field-from-large,
			.zarafa-mailcreatepanel-field-to,
			.zarafa-mailcreatepanel-field-cc,
			.zarafa-mailcreatepanel-field-bcc,
			.zarafa-mailcreatepanel-field-attachments,
			.zarafa-mailcreatepanel-field-editor {
				padding: 0 $padding-medium $padding-medium;
				min-height: $button-small-height;

				button {
					text-align: left;
				}
			}

			.zarafa-mailcreatepanel-extrainfo {
				padding: 0 $padding-medium $padding-medium $padding-medium;

				div {
					padding: $padding-medium;
					background: $light-blue;
					border: 1px solid $grey5;
				}
			}

			/* Override min-width to take account for the drop down entry */
			.zarafa-mailcreatepanel-field-from {
				button {
					min-width: 72px;
				}
			}

			.zarafa-mailcreatepanel-field-subject {
				padding: $padding-medium $padding-medium $padding-medium 2*$padding-medium;
				border-top: 1px solid $grey;
				border-bottom: 1px solid $grey;
				border-left: none;
				border-right: none;
				font-size: $font-size-large !important;
			}

			.zarafa-mailcreatepanel-field-attachments {
				padding-top: $padding-medium;
			}

			.zarafa-mailcreatepanel-field-editor {
				/* Prevent a double padding on the bottom of the panel */
				padding-bottom: 0;
				padding-left: 0;
				padding-right: 0;

				textarea {
					padding: $padding-extra-large;
					font-family: monospace !important;
				}
			}
		}
	}
}

/**********************************************************************
 * Appointment create panel & Task create panel
 *********************************************************************/
.k-appointmentcreatetab,
.k-taskgeneraltab {
	.x-form-item {
		margin-bottom: 0;
	}

	.zarafa-calendar-appointment-extrainfo {
		padding: $padding-medium $padding-medium 0;

		div {
			padding: $padding-medium;
			background: $light-blue;
		}
	}

	.k-field-to {
		padding: $padding-medium;
		min-height: $button-small-height;
		border-bottom: 1px solid $grey;

		button {
			text-align: left;
		}
	}

	.k-subject-panel,
	.k-createin-panel,
	.k-location-panel {
		padding: $padding-medium;
		border-bottom: 1px solid $grey;

		label {
			text-align: left;
			padding: 0 $padding-medium;
			line-height: $font-default-line-height;
			padding-top: ($button-small-height - $font-default-line-height)/2;
			width: $button-small-text-min-width - 2 * $padding-medium !important;
		}
	}

	.k-datetime-panel {
		table {
			width: 100%;
			border-collapse: collapse;
			white-space: nowrap;

			tr:nth-child(2) {
				border-bottom: 1px solid $grey;
			}

			td:first-child {
				width: 100px;

				td:first-child {
					width: auto;
				}
			}
		}
	}

	.k-date-panel {
		padding: $padding-medium;
		padding-right: 2 * $padding-extra-extra-large;

		.x-form-item:first-child {
			label {
				padding: 0 $padding-medium;
				line-height: $font-default-line-height;
				padding-top: ($button-small-height - $font-default-line-height)/2;
			}
		}

		.x-form-item:last-child {
			label {
				padding: 0 $padding-large;
				line-height: $font-default-line-height;
				padding-top: ($button-small-height - $font-default-line-height)/2;
				text-align: right;
			}
		}
	}

	.k-busystatus-panel,
	.k-label-panel,
	.k-status-panel,
	.k-priority-complete-panel {
		padding: $padding-medium;
		padding-left: $padding-extra-large;
		border-left: 1px solid $grey;
	}

	.k-reminder-panel,
	.k-taskrequestsettings {
		padding: $padding-medium;
		padding-left: $trigger-width - $padding-small;

		.x-form-check-wrap {
			line-height: $font-default-line-height;
		}
	}

	.k-taskrequestsettings {
		border-bottom: 1px solid $grey;

		.x-form-check-wrap {
			label {
				padding-right: $padding-extra-extra-large;
			}
		}
	}

	.k-taskinfopanel {
		padding: $padding-medium;
	}

	.k-extrainfopanel {
		padding: $padding-medium;
		padding-bottom: 0;
	}

	.filler {
		padding: $padding-medium;
		padding-left: $padding-extra-large;
		border-left: 1px solid $grey;
		height: $button-small-height;
	}

	.k-field-attachments {
		padding: $padding-medium;
	}

	.k-body-panel{
		textarea {
			border: none;
			border-top: 1px solid $grey;
		}
	}
}

.k-createin-combo {
	text-indent: 20px;
	background-position-x: 6px;
	background-position-y: 4px;
	background-repeat: no-repeat;
}

.k-createin-combo-list-svg {
	.x-combo-list-inner .x-combo-list-item svg {
		position:relative;
		top:2px;
		right:5px;
	}
}


.k-taskdetailtab {
	> .x-panel-bwrap {
		padding: $padding-medium;
	}

	.k-datepanel,
	.k-workpanel,
	.k-companypanel,
	.k-updatelistpanel {
		padding: $padding-medium;
	}

	.k-workpanel {
		.k-col1 {
			padding-left: $padding-extra-extra-large;
		}
	}
	
	.k-updatelistpanel {
		.x-btn {
			padding-top: $padding-medium;
		}	
	}
}

/**********************************************************************
 * Contact create panel
 *********************************************************************/
.zarafa-contactcreatepanel {
	/* the general tab */
	.zarafa-contactgeneraltab,
	.zarafa-contactdetailtab {
		> .x-panel-bwrap {
			> .x-panel-body {
				padding: $padding-medium;

				fieldset {
					border: none;

					legend {
						font-weight: bold;
					}

					.x-form-item {
						margin-bottom: $padding-medium;
					}

					.zarafa-compositefield.x-box-layout-ct {
						overflow: visible;
					}

					.zarafa-contactcreatepanel-field-attachments {
						padding: 0 $padding-medium $padding-medium;
						min-height: $button-small-height;

						button {
							text-align: left;
						}
					}

					.x-btn {

						.x-btn-small {
							width: 100%;

							em {
								display: block;

								button {
									text-align: left;
									width: 100%;
								}
							}
						}
					}
				}

				.zarafa-clear {
					padding-bottom: $padding-extra-extra-large;
				}
			}
		}
	}

	.zarafa-contactgeneraltab {
		.contact_photo_box_ct {
			width: 94px !important;
			height: 94px !important;
			background: $dark-grey2;
			box-shadow: inset 0px 0px 0px $padding-small $white;
			border : 1px solid $grey;
			padding: $padding-small;
			-webkit-touch-callout: none; /* iOS Safari */
			-webkit-user-select: none;   /* Chrome/Safari/Opera */
			-moz-user-select: none;      /* Firefox */
			-ms-user-select: none;       /* IE/Edge */
			user-select: none;           /* non-prefixed version, currently not supported by any browser */
		}
		.contact_photo_box {
			margin: $padding-small 0 0 $padding-small;
			width: 94px;
			height: 94px;
			cursor: pointer;
			background-size: contain;
			background-repeat: no-repeat;
			background-position: left top;
			background-color: $white;
		}
		.contact_photo_box:hover {
			border-color: $dark-grey2;
		}
		.default_contact_photo {
			background: url(../images/default_contact_profile.png) no-repeat center center;
		}
	}

}

/**********************************************************************
 * Distlist create panel
 *********************************************************************/
.zarafa-distlist-createpanel-memberstab {
	.buttons {
		td {
			padding-bottom: 3px; /*needed to show the shadow of the buttons*/
		}
	}
}


/* textarea duplicate border */
.zarafa-mailcreatepanel-field-editor.x-form-textarea {
	border-width: 1px 0px 0px;
}

/**********************************************************************
 * Task create panel
 *********************************************************************/
.zarafa-taskgeneraltab {
	.field-to {
		min-height: $button-small-height;
		border-bottom: 1px solid $grey;

		button {
			text-align: left;
		}
	}
	.zarafa-taskcreatepanel-field-attachments {
		padding-top: $padding-medium;
	}
}

/**********************************************************************
 * Meeting tracking tab
 *********************************************************************/
.k-trackingtab {

	.x-form-display-field {
		padding : 10px 10px 10px $padding-extra-large;
	}

	.x-panel-body {
		border-right: none;
		border-left: none;
		.x-grid3-row{
			border-left: none;
		}
	}
}
