tinymce.addI18n('de', {
"Paste Formatting Options": "Optionen f\u00FCr \"Formatierung einf\u00FCgen\"",
"Choose to keep or remove formatting in the pasted content.": "W\u00E4hlen Sie aus, ob die Formatierung in eingef\u00FCgten Inhalten beibehalten werden soll.",
"Keep Formatting": "Formatierung beibehalten",
"Remove Formatting": "Formatierung entfernen",
"Local Image Import": "Lokaler Bildimport",
"Trigger paste again from the keyboard to paste content with images.": "L\u00F6sen Sie das Einf\u00FCgen noch einmal mit der Tastatur aus, um Inhalt mit Bildern einzuf\u00FCgen.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Zum Importieren von Bildern aus Microsoft Office ist Adobe Flash erforderlich. <a href=\"http installieren://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Dr\u00FCcken Sie auf <span class=\"ephox-polish-help-kbd\">ESC</span>, um lokale Bilder zu ignorieren und mit der Bearbeitung fortzufahren.",
"Please wait...": "Bitte warten...",
"Your browser security settings may be preventing images from being imported.": "Eventuell verhindern die Sicherheitseinstellungen Ihres Browsers den Import von Bildern.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Eventuell verhindern die Sicherheitseinstellungen Ihres Browsers den Import von Bildern. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Weitere Informationen zum Einf\u00FCgen in Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari unterst\u00FCtzt nicht das direkte Einf\u00FCgen von Bildern. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Weitere Informationen zum Einf\u00FCgen von Bildern in Safari</a>",
"The images service was not found: (": "Der Bilddienst wurde nicht gefunden: (",
"Image failed to upload: (": "Fehler beim Hochladen des Bildes: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Einf\u00FCgen lokaler Bilder wurde deaktiviert. Lokale Bilder wurden aus dem eingef\u00FCgten Inhalt entfernt."
});