tinymce.addI18n('pt_PT', {
"Paste Formatting Options": "Op\u00E7\u00F5es de formata\u00E7\u00E3o de colagem",
"Choose to keep or remove formatting in the pasted content.": "Optar por manter ou remover a formata\u00E7\u00E3o do conte\u00FAdo colado.",
"Keep Formatting": "Manter formata\u00E7\u00E3o",
"Remove Formatting": "Remover formata\u00E7\u00E3o",
"Local Image Import": "Importa\u00E7\u00E3o de imagem local",
"Trigger paste again from the keyboard to paste content with images.": "Acione novamente a colagem a partir do teclado para colar conte\u00FAdo com imagens.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "\u00C9 necess\u00E1rio o Adobe Flash para importar imagens do Microsoft Office. Instale o <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Prima <span class=\"ephox-polish-help-kbd\">ESC</span> para ignorar as imagens locais e continuar a edi\u00E7\u00E3o.",
"Please wait...": "Aguarde...",
"Your browser security settings may be preventing images from being imported.": "As defini\u00E7\u00F5es de seguran\u00E7a do navegador podem estar a impedir a importa\u00E7\u00E3o das imagens.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "As defini\u00E7\u00F5es de seguran\u00E7a do navegador podem estar a impedir a importa\u00E7\u00E3o das imagens. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Mais informa\u00E7\u00F5es sobre colagem para o Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "O Safari n\u00E3o oferece suporte para a colagem direta de imagens. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Mais informa\u00E7\u00F5es sobre colagem de imagens para o Safari</a>",
"The images service was not found: (": "O servi\u00E7o de imagens n\u00E3o foi encontrado: (",
"Image failed to upload: (": "Falha ao carregar a imagem: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "A colagem da imagem local foi desativada. As imagens locais foram removidas do conte\u00FAdo colado."
});