tinymce.addI18n('es', {
"Paste Formatting Options": "Opciones para pegar formato",
"Choose to keep or remove formatting in the pasted content.": "Escoja entre conservar o quitar el formato del contenido pegado.",
"Keep Formatting": "Conservar formato",
"Remove Formatting": "Quitar formato",
"Local Image Import": "Importar imagen local",
"Trigger paste again from the keyboard to paste content with images.": "Activar de nuevo la acci\u00F3n de pegado desde el teclado para pegar el contenido con im\u00E1genes.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Es necesario tener Adobe Flash para importar im\u00E1genes desde Microsoft Office. Instalar <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Pulse <span class=\"ephox-polish-help-kbd\">ESC</span> para ignorar las im\u00E1genes locales y seguir editando.",
"Please wait...": "Espere...",
"Your browser security settings may be preventing images from being imported.": "Es posible que la configuraci\u00F3n de seguridad de su navegador est\u00E9 impidiendo que se importen las im\u00E1genes.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Es posible que la configuraci\u00F3n de seguridad de su navegador est\u00E9 impidiendo que se importen las im\u00E1genes. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">M\u00E1s informaci\u00F3n sobre el pegado en Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari no permite que se peguen las im\u00E1genes directamente. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">M\u00E1s informaci\u00F3n sobre el pegado de im\u00E1genes en Safari</a>",
"The images service was not found: (": "No se encuentra el servicio de im\u00E1genes: (",
"Image failed to upload: (": "No se ha cargado la imagen: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "La acci\u00F3n de pegado de im\u00E1genes locales ha sido desactivada. Se han eliminado im\u00E1genes locales del contenido pegado."
});