.arrow_down_l, .x-panel-collapsed .x-tool.x-tool-toggle, td.sort-desc .x-grid3-sort-icon, .x-tree-lines .x-tree-elbow-end-minus, .x-tree-lines .x-tree-elbow-minus, .preview-header-collapse-plus, .zarafa-rules-sequence-down {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc5JyBoZWlnaHQ9JzUnIHZpZXdCb3g9JzAgMCA1LjQwMDM5MDYgMyc+IDxwYXRoIHN0eWxlPSdmaWxsOiM0ZjRjNGQnIGQ9J00gMCAwIEwgMi42OTkyMTg4IDMgTCA1LjQwMDM5MDYgMCBMIDAgMCB6ICcgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_down_s, .x-btn .x-btn-split, .x-btn .x-btn-arrow, .x-form-trigger, .x-form-spinner-trigger, .x-item-disabled .x-form-trigger-over {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzMnIHZpZXdCb3g9JzAgMCA1LjQwMDM5MDYgMyc+IDxwYXRoIHN0eWxlPSdmaWxsOiM0ZjRjNGQnIGQ9J00gMCAwIEwgMi42OTkyMTg4IDMgTCA1LjQwMDM5MDYgMCBMIDAgMCB6ICcgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_down_s_white, .x-btn.x-btn.zarafa-action .x-btn-split, .x-btn.x-btn.zarafa-action .x-btn-arrow {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nMycgd2lkdGg9JzUnIHZpZXdCb3g9JzAgMCA1LjQwMDM5MDYgMyc+IDxwYXRoIHN0eWxlPSdmaWxsOiNmZmYnIGQ9J20wIDAgMi42OTkyIDMgMi43MDEyLTNoLTUuNDAwNHonLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_left_l, .light-background .zarafa-calendar-tab-icon-merge, .x-date-left a, .x-date-mp-ybtn a.x-date-mp-prev, .x-tool.x-tool-toggle.x-tool-collapse-west, .x-layout-collapsed .x-tool.x-tool-toggle.x-tool-expand-east, .x-panel-header .x-tool.x-tool-collapse-west, .x-layout-collapsed .x-tool.x-tool-expand-east, .x-tab-scroller-left {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknIHZpZXdCb3g9JzAgMCA1IDknPiA8cGF0aCBzdHlsZT0nZmlsbDojNGY0YzRkJyBkPSdNIDUsMCA1LDkgMCw0LjUgNSwwIFonIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_left_l_white, .zarafa-calendar-tab-icon-merge {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknIHZpZXdCb3g9JzAgMCA1IDknPiA8cGF0aCBzdHlsZT0nZmlsbDojZmZmZmZmJyBkPSdNIDUsMCA1LDkgMCw0LjUgNSwwIFonIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_menubar, .zarafa-maintabbar-maintab-active button.x-btn-text, .zarafa-maintabbar-maintab-active.x-btn-over button.x-btn-text, .zarafa-maintabbar-maintab-active.x-btn-over.x-btn-click button.x-btn-text {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMicgaGVpZ2h0PSc2JyB2aWV3Qm94PScwIDAgMTIgNic+IDxwYXRoIHN0eWxlPSdmaWxsOiNmZmZmZmYnIGQ9J00gNiAwIEwgMCA2IEwgMTIgNiBMIDYgMCB6ICcgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_right_l, .light-background .zarafa-calendar-tab-icon-separate, .x-date-right a, .x-date-mp-ybtn a, .x-tree-lines .x-tree-elbow-plus, .x-tree-lines .x-tree-elbow-end-plus, .x-menu-item-arrow, .x-tool.x-tool-toggle.x-tool-collapse-east, .x-layout-collapsed .x-tool.x-tool-toggle.x-tool-expand-west, .x-panel-header .x-tool.x-tool-collapse-east, .x-layout-collapsed .x-tool.x-tool-expand-west, .x-tab-scroller-right {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknIHZpZXdCb3g9JzAgMCA1IDknPiA8cGF0aCBzdHlsZT0nZmlsbDojNGY0YzRkJyBkPSdNIDAgMCBMIDAgOSBMIDUgNC41IEwgMCAwIHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_right_l_white, .zarafa-calendar-tab-icon-separate {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknIHZpZXdCb3g9JzAgMCA1IDknPiA8cGF0aCBzdHlsZT0nZmlsbDojZmZmZmZmJyBkPSdNIDAgMCBMIDAgOSBMIDUgNC41IEwgMCAwIHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_up_l, .x-tool.x-tool-toggle, td.sort-asc .x-grid3-sort-icon, .preview-header-collapse-minus, .zarafa-rules-sequence-up {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nNScgd2lkdGg9JzknIHZpZXdCb3g9JzAgMCA1LjQwMDM5MDYgMyc+IDxwYXRoIHN0eWxlPSdmaWxsOiM0ZjRjNGQnIGQ9J20wIDMgMi42OTkyLTMgMi43MDEyIDNoLTUuNDAwNHonLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_updown, .x-form-field-wrap .x-form-spinner-trigger {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nMTQnIHdpZHRoPSc1JyB2aWV3Qm94PScwIDAgNC45OTk5OTk4IDE0Jz4gPGcgdHJhbnNmb3JtPSd0cmFuc2xhdGUoMCAtMTAzOC40KSc+IDxwYXRoIHN0eWxlPSdmaWxsOiM0ZjRjNGQnIGQ9J20wIDEwNDkuNiAyLjQ5OTEgMi43Nzc2bDIuNTAwOS0yLjhoLTV6Jy8+IDxwYXRoIHN0eWxlPSdmaWxsOiM0ZjRjNGQnIGQ9J20wIDEwNDEuNCAyLjQ5OTEtMi43Nzc2IDIuNTAwOSAyLjc3NzZoLTV6Jy8+IDwvZz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross, .icon_remove, .x-zarafa-boxfield-item-close, .x-zarafa-boxfield-suggestion-cross, .light-background .zarafa-calendar-tab-icon-close, .x-freebusy-userrow-remove, .x-tab-strip .x-tab-strip-closable a.x-tab-strip-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc3JyBoZWlnaHQ9JzcnIHZpZXdCb3g9JzAgMCA3IDcnPiA8cGF0aCBzdHlsZT0nZmlsbDojNGY0YzRkJyBkPSdNIDAuNzIwNzAzMTIgMC4wMjM0Mzc1IEwgMC4wMTU2MjUgMC43MzA0Njg3NSBMIDIuNzkyOTY4OCAzLjUgTCAwLjAxNTYyNSA2LjI2OTUzMTIgTCAwLjcyMDcwMzEyIDYuOTc2NTYyNSBMIDMuNSA0LjIwNTA3ODEgTCA2LjI3OTI5NjkgNi45NzY1NjI1IEwgNi45ODQzNzUgNi4yNjk1MzEyIEwgNC4yMDcwMzEyIDMuNSBMIDYuOTg0Mzc1IDAuNzMwNDY4NzUgTCA2LjI3OTI5NjkgMC4wMjM0Mzc1IEwgMy41IDIuNzk0OTIxOSBMIDAuNzIwNzAzMTIgMC4wMjM0Mzc1IHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross_large, .x-tool-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMCcgaGVpZ2h0PScxMCcgdmlld0JveD0nMCAwIDEwIDEwJz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nTSAwLjcwODk4NDM4IDAgTCAwIDAuNzA4OTg0MzggTCA0LjI5MTAxNTYgNSBMIDAgOS4yOTEwMTU2IEwgMC43MDg5ODQzOCAxMCBMIDUgNS43MDg5ODQ0IEwgOS4yOTEwMTU2IDEwIEwgMTAgOS4yOTEwMTU2IEwgNS43MDg5ODQ0IDUgTCAxMCAwLjcwODk4NDM4IEwgOS4yOTEwMTU2IDAgTCA1IDQuMjkxMDE1NiBMIDAuNzA4OTg0MzggMCB6ICcgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross_white, .zarafa-calendar-tab-icon-close, .k-category-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc3JyBoZWlnaHQ9JzcnIHZpZXdCb3g9JzAgMCA3IDcnPiA8cGF0aCBzdHlsZT0nZmlsbDojZmZmZmZmJyBkPSdNIDAuNzIwNzAzMTIgMC4wMjM0Mzc1IEwgMC4wMTU2MjUgMC43MzA0Njg3NSBMIDIuNzkyOTY4OCAzLjUgTCAwLjAxNTYyNSA2LjI2OTUzMTIgTCAwLjcyMDcwMzEyIDYuOTc2NTYyNSBMIDMuNSA0LjIwNTA3ODEgTCA2LjI3OTI5NjkgNi45NzY1NjI1IEwgNi45ODQzNzUgNi4yNjk1MzEyIEwgNC4yMDcwMzEyIDMuNSBMIDYuOTg0Mzc1IDAuNzMwNDY4NzUgTCA2LjI3OTI5NjkgMC4wMjM0Mzc1IEwgMy41IDIuNzk0OTIxOSBMIDAuNzIwNzAzMTIgMC4wMjM0Mzc1IHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_tools_cogwheel, .x-tool.x-tool-gear {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMycgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nTSA2LjI1MTk1MzEgMC45OTgwNDY4OCBMIDYuMDk3NjU2MiAyLjg0MTc5NjkgQSA1LjUgNS41IDAgMCAwIDQuNDkwMjM0NCAzLjc3MTQ4NDQgTCAyLjgxNDQ1MzEgMi45ODI0MjE5IEwgMS4wNjQ0NTMxIDYuMDEzNjcxOSBMIDIuNTg1OTM3NSA3LjA3MDMxMjUgQSA1LjUgNS41IDAgMCAwIDIuNTAxOTUzMSA3Ljk5ODA0NjkgQSA1LjUgNS41IDAgMCAwIDIuNTgyMDMxMiA4LjkyOTY4NzUgTCAxLjA2NDQ1MzEgOS45ODI0MjE5IEwgMi44MTQ0NTMxIDEzLjAxNTYyNSBMIDQuNDg2MzI4MSAxMi4yMjg1MTYgQSA1LjUgNS41IDAgMCAwIDYuMDk3NjU2MiAxMy4xNTIzNDQgTCA2LjI0ODA0NjkgMTUuMDAxOTUzIEwgOS43NDgwNDY5IDE1LjAwMTk1MyBMIDkuOTAyMzQzOCAxMy4xNTgyMDMgQSA1LjUgNS41IDAgMCAwIDExLjUwOTc2NiAxMi4yMjg1MTYgTCAxMy4xODU1NDcgMTMuMDE3NTc4IEwgMTQuOTM1NTQ3IDkuOTg0Mzc1IEwgMTMuNDE0MDYyIDguOTI3NzM0NCBBIDUuNSA1LjUgMCAwIDAgMTMuNDk4MDQ3IDggQSA1LjUgNS41IDAgMCAwIDEzLjQxNzk2OSA3LjA2ODM1OTQgTCAxNC45MzU1NDcgNi4wMTU2MjUgTCAxMy4xODU1NDcgMi45ODQzNzUgTCAxMS41MTM2NzIgMy43NzE0ODQ0IEEgNS41IDUuNSAwIDAgMCA5LjkwMjM0MzggMi44NDc2NTYyIEwgOS43NTE5NTMxIDAuOTk4MDQ2ODggTCA2LjI1MTk1MzEgMC45OTgwNDY4OCB6IE0gOCA1IEEgMyAzIDAgMCAxIDExIDggQSAzIDMgMCAwIDEgOCAxMSBBIDMgMyAwIDAgMSA1IDggQSAzIDMgMCAwIDEgOCA1IHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_tools_minimize, .x-tool.x-tool-minimize {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMCcgaGVpZ2h0PScxMCcgdmlld0JveD0nMCAwIDEzIDEzJz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbS41IDEwYy0uMjc3IDAtLjUuMjIzLS41LjVzLjIyMy41LjUuNWgxMmMuMjc3IDAgLjUtLjIyMy41LS41cy0uMjIzLS41LS41LS41aC0xMnonLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_arrow_down_l {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGlkPSJzdmciIHdpZHRoPSI5IiBoZWlnaHQ9IjQ1LjAxNCIgdmVyc2lvbj0iMS4xIiB2aWV3Qm94PSIwIDAgMi4zODEzIDExLjkxIj4gPHBhdGggZD0ibTEuMTkxIDExLjkxdi0xMS4zNDMiIHN0eWxlPSJmaWxsOm5vbmU7c3Ryb2tlOiM0ZjRjNGQ7c3Ryb2tlLXdpZHRoOjAuMjY0NTgwMDE7c3Ryb2tlLW9wYWNpdHk6MSIvPiA8cGF0aCBkPSJtMC0uMDAwMjQ5ODkgMS4xOTAyIDEuMzIyOCAxLjE5MTEtMS4zMjI4eiIgc3R5bGU9ImZpbGw6IzRmNGM0ZDtzdHJva2Utd2lkdGg6LjQ0MDk1Ii8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_arrow_right_l {background-image: url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0nOS45OTk5JyBoZWlnaHQ9JzknIHZlcnNpb249JzEuMScgdmlld0JveD0nMCAwIDIuNjQ1OCAyLjM4MTMnIHhtbG5zPSdodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2Zyc+IDxwYXRoIGQ9J20tMS42NjY3ZS01IDEuMTloMS41ODc1JyBzdHlsZT0nZmlsbDpub25lO3N0cm9rZS13aWR0aDouMjY0NTg7c3Ryb2tlOiM0ZjRjNGQnLz4gPHBhdGggZD0nbTEuMzIyOSAwdjIuMzgxM2wxLjMyMjktMS4xOTA3eicgc3R5bGU9J2ZpbGw6IzRmNGM0ZDtzdHJva2Utd2lkdGg6LjI2NDU5Jy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_vertical, .line_vertical.x-grid3-row-selected {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScwLjk5OTk5NjQyJyBoZWlnaHQ9JzUwMCcgdmlld0JveD0nMCAwIDAuMjY0NTc5MDUgMTMyLjI5MDQ4Jz48cGF0aCBzdHlsZT0nZmlsbDpub25lO3N0cm9rZTojNGY0YzRkO3N0cm9rZS13aWR0aDowLjI2NDU4MDAxO3N0cm9rZS1vcGFjaXR5OjEnIGQ9J00gMC4xMzIyODk1MywwIFYgMTMyLjI5MDQ3Jy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.minimize_button {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMycgdmlld0JveD0nMCAwIDEzIDEzJz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nTSAwLjUgNiBDIDAuMjIzIDYgMCA2LjIyMyAwIDYuNSBDIDAgNi43NzcgMC4yMjMgNyAwLjUgNyBMIDEyLjUgNyBDIDEyLjc3NyA3IDEzIDYuNzc3IDEzIDYuNSBDIDEzIDYuMjIzIDEyLjc3NyA2IDEyLjUgNiBMIDAuNSA2IHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.plus, .x-tab-add, .x-tool.x-tool-plus, .zarafa-rules-add, .x-zarafa-boxfield-item-expand {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMycgdmlld0JveD0nMCAwIDEzIDEzJz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nTSA2LjUgMCBDIDYuMjIzIDAgNiAwLjIyMyA2IDAuNSBMIDYgNiBMIDAuNSA2IEMgMC4yMjMgNiAwIDYuMjIzIDAgNi41IEMgMCA2Ljc3NyAwLjIyMyA3IDAuNSA3IEwgNiA3IEwgNiAxMi41IEMgNiAxMi43NzcgNi4yMjMgMTMgNi41IDEzIEMgNi43NzcgMTMgNyAxMi43NzcgNyAxMi41IEwgNyA3IEwgMTIuNSA3IEMgMTIuNzc3IDcgMTMgNi43NzcgMTMgNi41IEMgMTMgNi4yMjMgMTIuNzc3IDYgMTIuNSA2IEwgNyA2IEwgNyAwLjUgQyA3IDAuMjIzIDYuNzc3IDAgNi41IDAgeiAnIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left, .x-tbar-page-prev {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSAxMS40ODQzNzUsMyBjIC0wLjA5MDMyLDAuMDAzMjYgLTAuMTc4MDYyLDAuMDMwOTMxIC0wLjI1MzkwNiwwLjA4MDA3OCBsIC03LjAwMDAwMDIsNC41IGMgLTAuMzA0NzI1MSwwLjE5NzAyNDEgLTAuMzA0NzI1MSwwLjY0MjgxOTcgMCwwLjgzOTg0MzggTCAxMS4yMzA0NjksMTIuOTE5OTIyIEMgMTEuNTYyODE3LDEzLjEzMjU0NSAxMS45OTg5NzMsMTIuODk0NTQxIDEyLDEyLjUgbCAwLC05IEMgMTIuMDAwMTA5LDMuMjE3NjUxMSAxMS43NjY1ODcsMi45OTEyMDU3IDExLjQ4NDM3NSwzIFonIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_outline, .x-item-disabled .x-tbar-page-prev {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSAxMS40ODQzNzUsMyBhIDAuNSwwLjUgMCAwIDAgLTAuMjUzOTA2LDAuMDgwMDc4IGwgLTcuMDAwMDAwMiw0LjUgYSAwLjUsMC41IDAgMCAwIDAsMC44Mzk4NDM4IEwgMTEuMjMwNDY5LDEyLjkxOTkyMiBBIDAuNSwwLjUgMCAwIDAgMTIsMTIuNSBsIDAsLTkgYSAwLjUsMC41IDAgMCAwIC0wLjUxNTYyNSwtMC41IHogTSAxMSw0LjQxNDA2MjEgMTEsMTEuNTg1OTM4IDUuNDIxODc1LDggMTEsNC40MTQwNjIxIFonIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_stop, .x-tbar-page-first {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSA0LDMgYyAtMC41NTQsMCAtMSwwLjQ0NiAtMSwxIEwgMywxMiBjIDAsMC41NTQgMC40NDYsMSAxLDEgbCAxLDAgYyAwLjU1NCwwIDEsLTAuNDQ2IDEsLTEgTCA2LDguOTE0MDYyMSAxMi4yMzA0NjksMTIuOTE5OTIyIEMgMTIuNTYyODE3LDEzLjEzMjU0NSAxMi45OTg5NzMsMTIuODk0NTQxIDEzLDEyLjUgbCAwLC05IGMgMS4wOWUtNCwtMC4yODIzNDg5IC0wLjIzMzQxMywtMC41MDg3OTQzIC0wLjUxNTYyNSwtMC41IC0wLjA5MDMyLDAuMDAzMjYgLTAuMTc4MDYyLDAuMDMwOTMxIC0wLjI1MzkwNiwwLjA4MDA3OCBMIDYsNy4wODU5MzcxIDYsNCBjIDAsLTAuNTU0IC0wLjQ0NiwtMSAtMSwtMSB6JyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_stop_outline, .x-item-disabled .x-tbar-page-first {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nTSA0IDMgQyAzLjQ0NiAzIDMgMy40NDYgMyA0IEwgMyAxMiBDIDMgMTIuNTU0IDMuNDQ2IDEzIDQgMTMgTCA1IDEzIEMgNS41NTQgMTMgNiAxMi41NTQgNiAxMiBMIDYgOC45MTQwNjI1IEwgMTIuMjMwNDY5IDEyLjkxOTkyMiBBIDAuNTAwMDUgMC41MDAwNSAwIDAgMCAxMyAxMi41IEwgMTMgMy41IEEgMC41MDAwNSAwLjUwMDA1IDAgMCAwIDEyLjQ4NDM3NSAzIEEgMC41MDAwNSAwLjUwMDA1IDAgMCAwIDEyLjIzMDQ2OSAzLjA4MDA3ODEgTCA2IDcuMDg1OTM3NSBMIDYgNCBDIDYgMy40NDYgNS41NTQgMyA1IDMgTCA0IDMgeiBNIDQgMy44MzAwNzgxIEwgNSAzLjgzMDA3ODEgTCA1IDEyIEwgNCAxMiBMIDQgMy44MzAwNzgxIHogTSAxMiA0LjQxNDA2MjUgTCAxMiAxMS41ODU5MzggTCA2LjQyMTg3NSA4IEwgMTIgNC40MTQwNjI1IHogJyAvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right, .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSA0LjUxNzU1LDMgYyAwLjA5MDMyLDAuMDAzMjYgMC4xNzgwNjIsMC4wMzA5MzEgMC4yNTM5MDYsMC4wODAwNzggbCA3LDQuNSBjIDAuMzA0NzI1LDAuMTk3MDI0MSAwLjMwNDcyNSwwLjY0MjgxOTcgMCwwLjgzOTg0MzggbCAtNyw0LjUwMDAwMDIgQyA0LjQzOTEwOCwxMy4xMzI1NDUgNC4wMDI5NTIsMTIuODk0NTQxIDQuMDAxOTI1LDEyLjUgbCAwLC05IEMgNC4wMDE4MTYsMy4yMTc2NTExIDQuMjM1MzM4LDIuOTkxMjA1NyA0LjUxNzU1LDMgWicgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_outline, .x-item-disabled .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSA0LjUxNzUwNzksMyBhIDAuNSwwLjUgMCAwIDEgMC4yNTM5MDYsMC4wODAwNzggbCA3LjAwMDAwMDEsNC41IGEgMC41LDAuNSAwIDAgMSAwLDAuODM5ODQzOCBMIDQuNzcxNDEzOSwxMi45MTk5MjIgQSAwLjUsMC41IDAgMCAxIDQuMDAxODgyOSwxMi41IGwgMCwtOSBBIDAuNSwwLjUgMCAwIDEgNC41MTc1MDc5LDMgWiBtIDAuNDg0Mzc1LDEuNDE0MDYyMSAwLDcuMTcxODc1OSBMIDEwLjU4MDAwOCw4IDUuMDAxODgyOSw0LjQxNDA2MjEgWicgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_stop, .x-tbar-page-last {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSAxMiwzIGMgMC41NTQsMCAxLDAuNDQ2IDEsMSBsIDAsOCBjIDAsMC41NTQgLTAuNDQ2LDEgLTEsMSBsIC0xLDAgYyAtMC41NTQsMCAtMSwtMC40NDYgLTEsLTEgTCAxMCw4LjkxNDA2MjEgMy43Njk1MzEsMTIuOTE5OTIyIEMgMy40MzcxODMsMTMuMTMyNTQ1IDMuMDAxMDI3LDEyLjg5NDU0MSAzLDEyLjUgbCAwLC05IEMgMi45OTk4OTEsMy4yMTc2NTExIDMuMjMzNDEzLDIuOTkxMjA1NyAzLjUxNTYyNSwzIDMuNjA1OTQ1LDMuMDAzMjYgMy42OTM2ODcsMy4wMzA5MzEgMy43Njk1MzEsMy4wODAwNzggTCAxMCw3LjA4NTkzNzEgMTAsNCBjIDAsLTAuNTU0IDAuNDQ2LC0xIDEsLTEgeicgLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_stop_outline, .x-item-disabled .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNicgdmlld0JveD0nMCAwIDE2IDE2Jz4gPHBhdGggc3R5bGU9J2ZpbGw6IzRmNGM0ZCcgZD0nbSAxMiwzIGMgMC41NTQsMCAxLDAuNDQ2IDEsMSBsIDAsOCBjIDAsMC41NTQgLTAuNDQ2LDEgLTEsMSBsIC0xLDAgYyAtMC41NTQsMCAtMSwtMC40NDYgLTEsLTEgTCAxMCw4LjkxNDA2MjUgMy43Njk1MzEsMTIuOTE5OTIyIEEgMC41MDAwNSwwLjUwMDA1IDAgMCAxIDMsMTIuNSBsIDAsLTkgQSAwLjUwMDA1LDAuNTAwMDUgMCAwIDEgMy41MTU2MjUsMyAwLjUwMDA1LDAuNTAwMDUgMCAwIDEgMy43Njk1MzEsMy4wODAwNzgxIEwgMTAsNy4wODU5Mzc1IDEwLDQgYyAwLC0wLjU1NCAwLjQ0NiwtMSAxLC0xIGwgMSwwIHogbSAwLDAuODMwMDc4MSAtMSwwIEwgMTEsMTIgMTIsMTIgMTIsMy44MzAwNzgxIFogTSA0LDQuNDE0MDYyNSA0LDExLjU4NTkzOCA5LjU3ODEyNSw4IDQsNC40MTQwNjI1IFonIC8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.circle_disc {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGhlaWdodD0iMTAwIiB3aWR0aD0iMTAwIiBpZD0ic3ZnIj4KICA8Y2lyY2xlIGN4PSI1MCIgY3k9IjUwIiByPSIyLjYiIHN0eWxlPSJzdHJva2U6IzUwNGQ0ZTsgc3Ryb2tlLXdpZHRoOjE7IGZpbGw6IzRmNGM0ZCIvPgogIFNvcnJ5LCB5b3VyIGJyb3dzZXIgZG9lcyBub3Qgc3VwcG9ydCBpbmxpbmUgU1ZHLiAgCjwvc3ZnPg) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_circle_end, .line_circle_end.x-grid3-row-selected {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGlkPSJzdmciIHdpZHRoPSI5IiBoZWlnaHQ9IjU1IiB2ZXJzaW9uPSIxLjEiIHZpZXdCb3g9IjAgMCAyLjM4MTMgMTEuOTEiPiA8cGF0aCBkPSJtMS4xOTEgMTEuOTF2LTExLjM0MyIgc3R5bGU9ImZpbGw6bm9uZTtzdHJva2U6IzRmNGM0ZDtzdHJva2Utd2lkdGg6MC4yNjQ1ODAwMTtzdHJva2Utb3BhY2l0eToxIi8+ICAgCiAgICAgPGNpcmNsZSB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGN4PSIxLjE5IiBjeT0iMTEuNSIgcj0iMC4yNjQ1ODAwMSIgc3R5bGU9InN0cm9rZTojNTA0ZDRlOyBzdHJva2Utd2lkdGg6MTsgZmlsbDojNGY0YzRkIi8+Cjwvc3ZnPg==)!important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_circle_l,.line_circle_l.x-grid3-row-selected {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIGlkPSJzdmciIHdpZHRoPSI5IiBoZWlnaHQ9IjUwMCIgdmlld0JveD0iMCAwIDAuMjY0NTc5MDUgMTMyLjI5MDQ4Ij4KPHBhdGggeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIiBzdHlsZT0iZmlsbDpub25lO3N0cm9rZTojNGY0YzRkO3N0cm9rZS13aWR0aDowLjI2NDU4MDAxO3N0cm9rZS1vcGFjaXR5OjEiIGQ9Ik0gMC4xMzIyODk1MywwIFYgMTMyLjI5MDQ3Ii8+CjxjaXJjbGUgY3g9IjAuMTYiIGN5PSIxMCIgcj0iMC4yNjQ1ODAwMSIgc3R5bGU9InN0cm9rZTojNTA0ZDRlOyBzdHJva2Utd2lkdGg6MTsgZmlsbDojNGY0YzRkIi8+Cgo8L3N2Zz4=)!important; background-repeat: no-repeat !important; background-position: center center !important;}