<?php

namespace Sabre\Xml;

/**
 * This class contains the version number for this package.
 *
 * @copyright Copyright (C) 2009-2015 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://sabre.io/license/
 */
class Version {

    /**
     * Full version number
     */
    const VERSION = '1.5.0';

}
