@page{size:auto;margin:0.5cm}@media print{thead{display:table-header-group}#pagemargin{min-height:100%;margin:0}}html,div,dl,dt,dd,h1,h2,h3,h4,h5,h6,pre,form,fieldset,input,p,blockquote,th,td{margin:0;padding:0}img,html{border:0}address,caption,cite,code,dfn,th,var{font-style:normal;font-weight:normal}caption,th{text-align:left}h1,h2,h3,h4,h5,h6{font-size:100%}q:before,q:after{content:''}.nowrap{white-space:nowrap}.wrap{white-space:normal}.fullwidth{width:100%}table{width:100%;text-align:left;font-size:11px;font-family:arial, tahoma, helvetica, sans-serif;border-collapse:collapse}table th{width:25%;font-weight:bold;padding:2px 3px 2px 0px}table td{padding:2px 3px 2px 0px}
/*# sourceMappingURL=print.record.css.map */
