// AbsoluteLayout

.@{prefix}-abs-layout {
	position: relative;
}

body .@{prefix}-abs-layout-item, .@{prefix}-abs-end {
	position: absolute;
}

.@{prefix}-abs-end {
	width: 1px; height: 1px;
}

.@{prefix}-container-body.@{prefix}-abs-layout {
	overflow: hidden;
}
