/* This is just a sample file with CSS rules. You should write your own @font-face declarations
 * to add support for your desired fonts.
 */

@font-face {
  font-family: 'Novecentowide Book';
  src: url("./ViewerJS/fonts/Novecentowide-Bold-webfont.eot");
  src: url("./ViewerJS/fonts/Novecentowide-Bold-webfont.eot?#iefix") format("embedded-opentype"),
  url("./ViewerJS/fonts/Novecentowide-Bold-webfont.woff") format("woff"),
  url("./fonts/Novecentowide-Bold-webfont.ttf") format("truetype"),
  url("./fonts/Novecentowide-Bold-webfont.svg#NovecentowideBookBold") format("svg");
  font-weight: normal;
  font-style: normal;
}

@font-face {
    font-family: 'exotica';
    src: url('./ViewerJS/fonts/Exotica-webfont.eot');
    src: url('./ViewerJS/fonts/Exotica-webfont.eot?#iefix') format('embedded-opentype'),
         url('./ViewerJS/fonts/Exotica-webfont.woff') format('woff'),
         url('./ViewerJS/fonts/Exotica-webfont.ttf') format('truetype'),
         url('./ViewerJS/fonts/Exotica-webfont.svg#exoticamedium') format('svg');
    font-weight: normal;
    font-style: normal;

}

