/* OWNCLOUD BACKEND STYLES */

/* The main icon for the backend*/
/* Naming convention: icon_<size>_<backend name> */
.icon_16_Owncloud {
	background-image: url(../icons/owncloud16.png)!important;
	background-repeat: no-repeat;
}

.icon_24_Owncloud {
	background-image: url(../icons/owncloud24.png)!important;
	background-repeat: no-repeat;
}

.icon_256_Owncloud {
	background-image: url(../icons/owncloud256.png)!important;
	background-repeat: no-repeat;
}

.icon_16_logo_Owncloud {
    background-image: url(../icons/owncloud_logo16.png)!important;
    background-repeat: no-repeat;
}

.shareicon_16_user {
	background-image: url(../icons/usericon.png)!important;
	background-repeat: no-repeat;
	background-position: center;
}

.shareicon_16_group {
	background-image: url(../icons/groupicon.png)!important;
	background-repeat: no-repeat;
	background-position: center;
}

.shareicon_16_yes {
	background-image: url(../icons/icon_yes.png)!important;
	background-repeat: no-repeat;
	background-position: center;
}

.shareicon_16_no {
	background-image: url(../icons/icon_no.png)!important;
	background-repeat: no-repeat;
	background-position: center;
}

.icon_copy_clipboard {
	background-image: url(../icons/clipboard.png)!important;
	background-repeat: no-repeat;
	background-position: center;
}

/**
 * USER GROUP PREDICTOR FIELD
 */

.ugpredic_search_item {
	font:normal 11px tahoma, arial, helvetica, sans-serif;
	padding:3px 10px 3px 10px;
	border:1px solid #fff;
	border-bottom:1px solid #eeeeee;
	white-space:normal;
	color:#555;
}
.ugpredic_search_item h3 {
	display:block;
	font:inherit;
	font-weight:bold;
	color:#222;
}

.ugpredic_search_item h3 span {
	float: left;
	padding: 0 10px 0px 0px;
	font-weight:normal;
	width: 20px;
	display:block;
	clear:none;
}

/**
 * Checkboxes
 */

input[name=permissionShare],
input[name=permissionChange],
input[name=permissionCreate],
input[name=permissionDelete] {
	margin-top: 5px;
}

