<?php

require_once(__DIR__ . '/class.ZarafaException.php');

if (!class_exists('JSONException')) {
	/**
	 * Class will be used for exceptions that are generated by json_decode_data
	 * in server/util.php
	 * getCode() method will return actual JSON error constant.
	 */
	class JSONException extends ZarafaException
	{
	}
}
?>
