tinymce.addI18n('hu_HU', {
"Paste Formatting Options": "Beilleszt\u00E9s form\u00E1z\u00E1si be\u00E1ll\u00EDt\u00E1sai",
"Choose to keep or remove formatting in the pasted content.": "V\u00E1lasszon, hogy meg szeretn\u00E9 \u0151rizni vagy el szeretn\u00E9 t\u00E1vol\u00EDtani a beillesztett tartalom form\u00E1z\u00E1s\u00E1t.",
"Keep Formatting": "Form\u00E1z\u00E1s meg\u0151rz\u00E9se",
"Remove Formatting": "Form\u00E1z\u00E1s elt\u00E1vol\u00EDt\u00E1sa",
"Local Image Import": "Helyi k\u00E9p import\u00E1l\u00E1sa",
"Trigger paste again from the keyboard to paste content with images.": "Haszn\u00E1lja m\u00E9g egyszer a beilleszt\u00E9s billenty\u0171parancs\u00E1t a tartalom k\u00E9pekkel egy\u00FCtt t\u00F6rt\u00E9n\u0151 beilleszt\u00E9s\u00E9hez.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash sz\u00FCks\u00E9ges a Microsoft Office-b\u00F3l t\u00F6rt\u00E9n\u0151 import\u00E1l\u00E1shoz. Telep\u00EDtse: <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Nyomja meg az <span class=\"ephox-polish-help-kbd\">ESC</span> billenty\u0171t a helyi k\u00E9pek mell\u0151z\u00E9s\u00E9hez \u00E9s a szerkeszt\u00E9s folytat\u00E1s\u00E1hoz.",
"Please wait...": "K\u00E9rem, v\u00E1rjon...",
"Your browser security settings may be preventing images from being imported.": "A b\u00F6ng\u00E9sz\u0151 biztons\u00E1gi be\u00E1ll\u00EDt\u00E1sai lehet, hogy megakad\u00E1lyozz\u00E1k a k\u00E9pek import\u00E1l\u00E1s\u00E1t.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "A b\u00F6ng\u00E9sz\u0151 biztons\u00E1gi be\u00E1ll\u00EDt\u00E1sai lehet, hogy megakad\u00E1lyozz\u00E1k a k\u00E9pek import\u00E1l\u00E1s\u00E1t. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Tov\u00E1bbi tudnival\u00F3k a beilleszt\u00E9sr\u0151l Safari rendszeren</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "A Safari nem t\u00E1mogatja a k\u00E9pek k\u00F6zvetlen beilleszt\u00E9s\u00E9t. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Tov\u00E1bbi tudnival\u00F3k a k\u00E9pek beilleszt\u00E9s\u00E9r\u0151l Safari rendszeren</a>",
"The images service was not found: (": "A k\u00E9pszolg\u00E1ltat\u00E1s nem tal\u00E1lhat\u00F3: (",
"Image failed to upload: (": "A k\u00E9p felt\u00F6lt\u00E9se sikertelen: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "A helyi k\u00E9pek beilleszt\u00E9s\u00E9nek lehet\u0151s\u00E9ge letiltva. A helyi k\u00E9pek el lettek t\u00E1vol\u00EDtva a beillesztett tartalomb\u00F3l."
});