tinymce.addI18n('fi', {
"Paste Formatting Options": "Liitt\u00E4misen muotoiluvalinnat",
"Choose to keep or remove formatting in the pasted content.": "Valitse, pidet\u00E4\u00E4nk\u00F6 vai poistetaanko liitetyn sis\u00E4ll\u00F6n muotoilu.",
"Keep Formatting": "Pid\u00E4 muotoilu",
"Remove Formatting": "Poista muotoilu",
"Local Image Import": "Paikallisen kuvan tuonti",
"Trigger paste again from the keyboard to paste content with images.": "Laukaise liit\u00E4 uudestaan n\u00E4pp\u00E4imist\u00F6lt\u00E4 kuvia sis\u00E4lt\u00E4vn sis\u00E4ll\u00F6n liitt\u00E4miseksi.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash tarvitaan tuomaan kuvia Microsoft Officesta. Asenna the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Paina <span class=\"ephox-polish-help-kbd\">ESC</span> sivuuttaaksesi paikalliset kuvat ja jatkaaksesi muokkausta.",
"Please wait...": "Odota...",
"Your browser security settings may be preventing images from being imported.": "Selaimesi tietoturva-asetukset saattavat est\u00E4\u00E4 kuvien tuonnin.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Selaimesi tietoturva-asetukset saattavat est\u00E4\u00E4 kuvien tuonnin. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Lis\u00E4tietoja liit\u00E4-toiminnosta Safaria varten</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari ei tue kuvien suoraa liitt\u00E4mist\u00E4. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Lis\u00E4tietoja kuvien liitt\u00E4misest\u00E4 Safaria varten</a>",
"The images service was not found: (": "Kuvapalvelua ei l\u00F6ytynyt: (",
"Image failed to upload: (": "Kuvan lataaminen ep\u00E4onnistui: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Paikallisen kuvan liitt\u00E4minen on pois k\u00E4yt\u00F6st\u00E4. Paikalliset kuvat on poistettu liitetyst\u00E4 sis\u00E4ll\u00F6st\u00E4."
});