tinymce.addI18n('nl', {
"Paste Formatting Options": "Opties voor opmaak plakken",
"Choose to keep or remove formatting in the pasted content.": "Kies of u de opmaak van de geplakte inhoud wilt behouden of verwijderen.",
"Keep Formatting": "Opmaak behouden",
"Remove Formatting": "Opmaak verwijderen",
"Local Image Import": "Importeren van lokale afbeelding",
"Trigger paste again from the keyboard to paste content with images.": "Druk opnieuw op Ctrl+V als u de inhoud inclusief afbeeldingen wilt plakken.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Voor het importeren van afbeeldingen uit Microsoft Office is Adobe Flash vereist. Installeer de <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Druk op <span class=\"ephox-polish-help-kbd\">Esc</span> als u lokale afbeeldingen wilt negeren en door wilt gaan met bewerken.",
"Please wait...": "Even wachten...",
"Your browser security settings may be preventing images from being imported.": "Afbeeldingen worden mogelijk niet ge\u00EFmporteerd vanwege de beveiligingsinstellingen van uw browser.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Afbeeldingen worden mogelijk niet ge\u00EFmporteerd vanwege de beveiligingsinstellingen van uw browser. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Meer informatie over plakken voor Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Het rechtstreeks plakken van afbeeldingen wordt niet ondersteund door Safari. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Meer informatie over het plakken van afbeeldingen voor Safari</a>",
"The images service was not found: (": "De service Afbeeldingen is niet gevonden: (",
"Image failed to upload: (": "Uploaden van afbeelding mislukt: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Lokale afbeelding plakken is gedeactiveerd. Lokale afbeeldingen zijn uit geplakte inhoud verwijderd."
});