tinymce.addI18n('zh_CN', {
"Paste Formatting Options": "\u7C98\u8D34\u683C\u5F0F\u9009\u9879",
"Choose to keep or remove formatting in the pasted content.": "\u9009\u62E9\u5728\u7C98\u8D34\u7684\u5185\u5BB9\u4FDD\u7559\u6216\u79FB\u9664\u683C\u5F0F\u3002",
"Keep Formatting": "\u4FDD\u6301\u683C\u5F0F",
"Remove Formatting": "\u79FB\u9664\u683C\u5F0F",
"Local Image Import": "\u5BFC\u5165\u672C\u5730\u56FE\u50CF",
"Trigger paste again from the keyboard to paste content with images.": "\u518D\u6B21\u4ECE\u952E\u76D8\u89E6\u53D1\u7C98\u8D34\u64CD\u4F5C\uFF0C\u7C98\u8D34\u542B\u56FE\u50CF\u7684\u5185\u5BB9\u3002",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "\u9700\u8981Adobe Flash\u624D\u80FD\u4ECE\u5FAE\u8F6F\u529E\u516C\u8F6F\u4EF6\u5BFC\u5165\u56FE\u50CF\u3002 \u5B89\u88C5<a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "\u6309<span class=\"ephox-polish-help-kbd\">ESC</span>\u5FFD\u7565\u672C\u5730\u56FE\u50CF\u5E76\u7EE7\u7EED\u7F16\u8F91",
"Please wait...": "\u8BF7\u7A0D\u7B49......",
"Your browser security settings may be preventing images from being imported.": "\u6D4F\u89C8\u5668\u7684\u5B89\u5168\u8BBE\u7F6E\u53EF\u80FD\u963B\u6B62\u4E86\u56FE\u50CF\u8F93\u5165\u3002",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "\u6D4F\u89C8\u5668\u7684\u5B89\u5168\u8BBE\u7F6E\u53EF\u80FD\u963B\u6B62\u4E86\u56FE\u50CF\u8F93\u5165\u3002 <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">\u66F4\u591A\u5173\u4E8ESafari\u7C98\u8D34\u64CD\u4F5C\u7684\u4FE1\u606F</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari\u6D4F\u89C8\u5668\u4E0D\u652F\u6301\u76F4\u63A5\u7C98\u8D34\u56FE\u50CF\u3002 <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">\u66F4\u591A\u5173\u4E8ESafari\u56FE\u50CF\u7C98\u8D34\u7684\u4FE1\u606F</a>",
"The images service was not found: (": "\u627E\u4E0D\u5230\u56FE\u50CF\u670D\u52A1\uFF1A(",
"Image failed to upload: (": "\u56FE\u50CF\u4E0A\u4F20\u5931\u8D25\uFF1A(",
").": ")\u3002",
"Local image paste has been disabled. Local images have been removed from pasted content.": "\u5C40\u90E8\u56FE\u50CF\u7C98\u8D34\u5DF2\u88AB\u7981\u7528\u3002 \u5C40\u90E8\u56FE\u50CF\u5DF2\u4ECE\u7C98\u8D34\u5185\u5BB9\u5904\u5220\u9664\u3002"
});