tinymce.addI18n('he_IL', {
"Paste Formatting Options": "\u05D0\u05E4\u05E9\u05E8\u05D5\u05D9\u05D5\u05EA \u05E2\u05D9\u05E6\u05D5\u05D1 \u05D4\u05D3\u05D1\u05E7\u05D4",
"Choose to keep or remove formatting in the pasted content.": "\u05D1\u05D7\u05E8 \u05D0\u05DD \u05DC\u05E9\u05DE\u05D5\u05E8 \u05D0\u05D5 \u05DC\u05D4\u05E1\u05D9\u05E8 \u05D0\u05EA \u05D4\u05E2\u05D9\u05E6\u05D5\u05D1 \u05D1\u05EA\u05D5\u05DB\u05DF \u05D4\u05DE\u05D5\u05D3\u05D1\u05E7.",
"Keep Formatting": "\u05E9\u05DE\u05D5\u05E8 \u05E2\u05D9\u05E6\u05D5\u05D1",
"Remove Formatting": "\u05D4\u05E1\u05E8 \u05E2\u05D9\u05E6\u05D5\u05D1",
"Local Image Import": "\u05D9\u05D9\u05D1\u05D5\u05D0 \u05EA\u05DE\u05D5\u05E0\u05D4 \u05DE\u05E7\u05D5\u05DE\u05D9\u05EA",
"Trigger paste again from the keyboard to paste content with images.": "\u05D1\u05E6\u05E2 \u05E9\u05D5\u05D1 \u05D0\u05EA \u05E4\u05E2\u05D5\u05DC\u05EA \u05D4\u05D4\u05D3\u05D1\u05E7\u05D4 \u05DE\u05D4\u05DE\u05E7\u05DC\u05D3\u05EA \u05DB\u05D3\u05D9 \u05DC\u05D4\u05D3\u05D1\u05D9\u05E7 \u05D0\u05EA \u05D4\u05EA\u05D5\u05DB\u05DF \u05E2\u05DD \u05D4\u05EA\u05DE\u05D5\u05E0\u05D5\u05EA.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Adobe Flash \u05E0\u05D3\u05E8\u05E9 \u05DB\u05D3\u05D9 \u05DC\u05D9\u05D9\u05D1\u05D0 \u05EA\u05DE\u05D5\u05E0\u05D5\u05EA \u05DE-Microsoft Office. \u05D4\u05EA\u05E7\u05DF \u05D0\u05EA <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "\u05D4\u05E7\u05E9 \u05E2\u05DC <span class=\"ephox-polish-help-kbd\">ESC</span> \u05DB\u05D3\u05D9 \u05DC\u05D4\u05EA\u05E2\u05DC\u05DD \u05DE\u05D4\u05EA\u05DE\u05D5\u05E0\u05D5\u05EA \u05D4\u05DE\u05E7\u05D5\u05DE\u05D9\u05D5\u05EA \u05D5\u05DC\u05D4\u05DE\u05E9\u05D9\u05DA \u05D1\u05E2\u05E8\u05D9\u05DB\u05D4.",
"Please wait...": "\u05D0\u05E0\u05D0 \u05D4\u05DE\u05EA\u05DF...",
"Your browser security settings may be preventing images from being imported.": "\u05D9\u05D9\u05EA\u05DB\u05DF \u05E9\u05D4\u05D2\u05D3\u05E8\u05D5\u05EA \u05D4\u05D0\u05D1\u05D8\u05D7\u05D4 \u05E9\u05DC \u05D4\u05D3\u05E4\u05D3\u05E4\u05DF \u05DE\u05D5\u05E0\u05E2\u05D5\u05EA \u05D9\u05D9\u05D1\u05D5\u05D0 \u05E9\u05DC \u05EA\u05DE\u05D5\u05E0\u05D5\u05EA.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "\u05D9\u05D9\u05EA\u05DB\u05DF \u05E9\u05D4\u05D2\u05D3\u05E8\u05D5\u05EA \u05D4\u05D0\u05D1\u05D8\u05D7\u05D4 \u05E9\u05DC \u05D4\u05D3\u05E4\u05D3\u05E4\u05DF \u05DE\u05D5\u05E0\u05E2\u05D5\u05EA \u05D9\u05D9\u05D1\u05D5\u05D0 \u05E9\u05DC \u05EA\u05DE\u05D5\u05E0\u05D5\u05EA. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">\u05DE\u05D9\u05D3\u05E2 \u05E0\u05D5\u05E1\u05E3 \u05DC\u05D2\u05D1\u05D9 \u05D4\u05D3\u05D1\u05E7\u05D4 \u05D1-Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Safari \u05D0\u05D9\u05E0\u05D5 \u05EA\u05D5\u05DE\u05DA \u05D1\u05D4\u05D3\u05D1\u05E7\u05D4 \u05D9\u05E9\u05D9\u05E8\u05D4 \u05E9\u05DC \u05EA\u05DE\u05D5\u05E0\u05D5\u05EA. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">\u05DE\u05D9\u05D3\u05E2 \u05E0\u05D5\u05E1\u05E3 \u05DC\u05D2\u05D1\u05D9 \u05D4\u05D3\u05D1\u05E7\u05EA \u05EA\u05DE\u05D5\u05E0\u05D5\u05EA \u05D1-Safari</a>",
"The images service was not found: (": "\u05E9\u05D9\u05E8\u05D5\u05EA \u05D4\u05EA\u05DE\u05D5\u05E0\u05D5\u05EA \u05DC\u05D0 \u05E0\u05DE\u05E6\u05D0: (",
"Image failed to upload: (": "\u05D4\u05E2\u05DC\u05D0\u05EA \u05D4\u05EA\u05DE\u05D5\u05E0\u05D4 \u05E0\u05DB\u05E9\u05DC\u05D4: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "\u05D4\u05D3\u05D1\u05E7\u05EA \u05D4\u05EA\u05DE\u05D5\u05E0\u05D4 \u05D4\u05DE\u05E7\u05D5\u05DE\u05D9\u05EA \u05D4\u05D5\u05E9\u05D1\u05EA\u05D4. \u05D4\u05EA\u05DE\u05D5\u05E0\u05D5\u05EA \u05D4\u05DE\u05E7\u05D5\u05DE\u05D9\u05D5\u05EA \u05D4\u05D5\u05E1\u05E8\u05D5 \u05DE\u05D4\u05EA\u05D5\u05DB\u05DF \u05D4\u05DE\u05D5\u05D3\u05D1\u05E7."
});