p,h1,h2,h3{font-family:arial, tahoma, helvetica, sans-serif}p.intro{font-size:1.25em}a{color:#3F51B5}body{background:url("../../images/loginbg_daybreak.jpg") no-repeat center center/cover}body :after{content:'';position:absolute;top:0;bottom:0;left:0;right:0;margin:0;padding:0;background:url("../../images/background.svg") center/cover no-repeat;z-index:-1}body .wrapper{width:50%;margin:auto;margin-top:5%;background-color:#fff;box-shadow:0 16px 24px 2px rgba(0,0,0,0.14),0 6px 30px 5px rgba(0,0,0,0.12),0 8px 10px -5px rgba(0,0,0,0.2)}body .wrapper .header{background-color:#c0392b;color:#fff;height:15%;display:flex;justify-content:center;align-items:center}body .wrapper .action{min-height:30%;padding:1.25em;color:#616161}body .wrapper .disable{min-height:10%;padding:1.25em;color:#616161}
/*# sourceMappingURL=configcheck.css.map */
