/* Pop-up notifications */
.zarafa-notifier-eventdropdown-item,
.zarafa-notifier-popup-item,
.zarafa-notifier-dropdown-item {
	background-color : #f4f4f4;
	text-align : left;
	display: table;
	width: 100%;
	box-shadow : 0 1px 1px $white;
	border-style : solid;
	border-color : #ccc;

	.left {
		display: table-cell;
		width: 40px;
		padding: 5px;
		box-sizing: border-box;
		vertical-align: middle;
	}

	.right {
		display: table-cell;
		text-align: left;
		padding: 10px 10px 10px 0px;
	}
}

.zarafa-notifier-dropdown-item,
.zarafa-notifier-eventdropdown-item
{
	border-width : 0 1px 1px 1px;
}
.zarafa-notifier-popup-item
{
	border-width : 1px 0 0 1px;
}

.zarafa-notifier-dropdown-item-title,
.zarafa-notifier-eventdropdown-item-title,
.zarafa-notifier-popup-item-title
{
	font-weight : bold;
}
.zarafa-notifier-container
{
	position : absolute;
	z-index : 1000000000;
	width : 356px;
}

.zarafa-slide-notifier-icon
{
	height: 30px;
}

.zarafa-slide-notifier-info,
.zarafa-slide-notifier-warning,
.zarafa-slide-notifier-error {
	.icon {
		height: 30px;
		width: 30px;
	}
}

.zarafa-slide-notifier-error {
	background-color : #f4eaea;
}

.zarafa-slide-notifier-warning {
	background-color : #f4eac0;
}

.zarafa-slide-notifier-info {
	background-color : #eaf4ea;
}

/* Create a normal formatted list in the Missing Default Folder notifier */
.zarafa-slide-notifier-error-hierarchy-defaultfolder ul {
	list-style: disc;
	padding-left: 40px;
}
