Ext.namespace("Zarafa.plugins.files.backend.SMB");Zarafa.plugins.files.backend.SMB.SMBBackend=Ext.extend(Zarafa.core.Plugin,{constructor:function(a){a=a||{};Zarafa.plugins.files.backend.SMB.SMBBackend.superclass.constructor.call(this,a)},initPlugin:function(){Zarafa.plugins.files.backend.SMB.SMBBackend.superclass.initPlugin.apply(this,arguments)}});
Zarafa.onReady(function(){container.registerPlugin(new Zarafa.core.PluginMetaData({name:"filesbackendSMB",displayName:dgettext("plugin_filesbackendSMB","Files: SMB/CIFS Backend"),allowUserDisable:!1,pluginConstructor:Zarafa.plugins.files.backend.SMB.SMBBackend}))});//# sourceMappingURL=filesbackendSMB.js.map
