// Tooltip

.@{prefix}-tooltip {
	position: absolute;
	padding: 5px;
	.opacity(0.8);
}

.@{prefix}-tooltip-inner {
	font-size: @tooltip-font-size;
	background-color: @tooltip-bg;
	color: @tooltip-text;
	max-width: 200px;
	padding: 5px 8px 4px 8px;
	text-align: center;
	white-space: normal;
}

.@{prefix}-tooltip-inner {
	.border-radius(3px);
}

.@{prefix}-tooltip-inner {
	.box-shadow(0 0 5px @tooltip-bg);
}

.@{prefix}-tooltip-arrow {
	position: absolute;
	width: 0;
	height: 0;
	line-height: 0;
	border: 5px dashed @tooltip-bg;
}

.@{prefix}-tooltip-arrow-n {
	border-bottom-color: @tooltip-bg;
}

.@{prefix}-tooltip-arrow-s {
	border-top-color: @tooltip-bg;
}

.@{prefix}-tooltip-arrow-e {
	border-left-color: @tooltip-bg;
}

.@{prefix}-tooltip-arrow-w {
	border-right-color: @tooltip-bg;
}

.@{prefix}-tooltip-nw, .@{prefix}-tooltip-sw {
	margin-left: -14px;
}

.@{prefix}-tooltip-n .@{prefix}-tooltip-arrow {
	top: 0px;
	left: 50%;
	margin-left: -5px;
	border-bottom-style: solid;
	border-top: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-nw .@{prefix}-tooltip-arrow {
	top: 0;
	left: 10px;
	border-bottom-style: solid;
	border-top: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-ne .@{prefix}-tooltip-arrow {
	top: 0;
	right: 10px;
	border-bottom-style: solid;
	border-top: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-s .@{prefix}-tooltip-arrow {
	bottom: 0;
	left: 50%;
	margin-left: -5px;
	border-top-style: solid;
	border-bottom: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-sw .@{prefix}-tooltip-arrow {
	bottom: 0;
	left: 10px;
	border-top-style: solid;
	border-bottom: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-se .@{prefix}-tooltip-arrow {
	bottom: 0;
	right: 10px;
	border-top-style: solid;
	border-bottom: none;
	border-left-color: transparent;
	border-right-color: transparent;
}

.@{prefix}-tooltip-e .@{prefix}-tooltip-arrow {
	right: 0;
	top: 50%;
	margin-top: -5px;
	border-left-style: solid;
	border-right: none;
	border-top-color: transparent;
	border-bottom-color: transparent;
}

.@{prefix}-tooltip-w .@{prefix}-tooltip-arrow {
	left: 0;
	top: 50%;
	margin-top: -5px;
	border-right-style: solid;
	border-left: none;
	border-top-color: transparent;
	border-bottom-color: transparent;
}
