tinymce.addI18n('pl', {
"Paste Formatting Options": "Wklej opcje formatowania",
"Choose to keep or remove formatting in the pasted content.": "Wybierz opcj\u0119 zachowywania lub usuwania formatowania we wklejanej tre\u015Bci.",
"Keep Formatting": "Zachowaj formatowanie",
"Remove Formatting": "Usu\u0144 formatowanie",
"Local Image Import": "Importuj obraz lokalny",
"Trigger paste again from the keyboard to paste content with images.": "Aktywuj ponownie wklejanie za pomoc\u0105 klawiatury, aby wklei\u0107 tre\u015B\u0107 wraz z obrazami.",
"Adobe Flash is required to import images from Microsoft Office. Install the <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.": "Do zaimportowania obraz\u00F3w z programu Microsoft Office wymagany jest program Adobe Flash. Zainstaluj <a href=\"http://get.adobe.com/flashplayer/\" target=\"_blank\">Adobe Flash Player</a>.",
"Press <span class=\"ephox-polish-help-kbd\">ESC</span> to ignore local images and continue editing.": "Naci\u015Bnij klawisz <span class=\"ephox-polish-help-kbd\">ESC</span> aby zignorowa\u0107 obrazy lokalne i kontynuowa\u0107 edycj\u0119.",
"Please wait...": "Prosz\u0119 czeka\u0107...",
"Your browser security settings may be preventing images from being imported.": "Ustawienia zabezpiecze\u0144 przegl\u0105darki mog\u0105 uniemo\u017Cliwi\u0107 zaimportowanie obraz\u00F3w.",
"Your browser security settings may be preventing images from being imported. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">More information on paste for Safari</a>": "Ustawienia zabezpiecze\u0144 przegl\u0105darki mog\u0105 uniemo\u017Cliwi\u0107 zaimportowanie obraz\u00F3w. <a href=\"https://support.ephox.com/entries/59328357-Safari-6-1-and-7-Flash-Sandboxing\" style=\"text-decoration: underline\">Wi\u0119cej informacji o wklejaniu do przegl\u0105darki Safari</a>",
"Safari does not support direct paste of images. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">More information on image pasting for Safari</a>": "Przegl\u0105darka Safari nie obs\u0142uguje bezpo\u015Bredniego wklejania obraz\u00F3w. <a href=\"https://support.ephox.com/entries/88543243-Safari-Direct-paste-of-images-does-not-work\" style=\"text-decoration: underline\">Wi\u0119cej informacji o wklejaniu obraz\u00F3w do przegl\u0105darki Safari</a>",
"The images service was not found: (": "Nie znaleziono us\u0142ugi obraz\u00F3w: (",
"Image failed to upload: (": "Nie powiod\u0142o si\u0119 przekazanie obrazu: (",
").": ").",
"Local image paste has been disabled. Local images have been removed from pasted content.": "Wklejanie obraz\u00F3w lokalnych zosta\u0142o wy\u0142\u0105czone. Obrazy lokalne zosta\u0142y usuni\u0119te z wklejonej tre\u015Bci."
});