/* import the base */
@import 'base/_colors';

@import 'base/_sizes';

@import 'base/_fonts';

/***************************************************************
 * _icons.scss contains the 'app icons' of Kopano WebApp, like
 * arrows and icons used for tools. They will always be
 * available and don't have to be implemented by icon sets.
 * This file is auto-generated by running `npm run app-icons`
 * Please do not forget to run compass when you have changed
 * this file!
 ***************************************************************/
@import 'base/_icons';

@import 'base/_layout';

/* import the components general styles */

@import 'components/_buttons';

@import 'components/_panels';

@import 'components/_formelements';

@import 'components/_grids';

@import 'components/_toolbars';

@import 'components/_menus';

/* import the component specific styles */

@import 'components/_mainmenubar';

@import 'components/_maintoolbar';

@import 'components/_tabs';

@import 'components/_boxfields';

@import 'components/_hierarchy';

@import 'components/_preview_panel';

@import 'components/_createitempanels';

@import 'components/_calendar';

@import 'components/_contacts';

@import 'components/_popups';

@import 'components/_scheduling';

@import 'components/_search';

@import 'components/_notes';

@import 'components/_settings';

@import 'components/_datepickers';

@import 'components/_colorpicker';

@import 'components/_welcome';

@import 'components/_categories';

@import 'components/_rules';

@import 'components/_notifications';

@import 'components/_widgets';

@import 'components/_loading_mask';

@import 'components/_tooltip';

@import 'components/_tinymce';

@import 'components/_presence';

@import 'components/_whatsnew';

/*
 * To make sure that the tinymce skin styling is loaded before the editor
 * is rendered, we import it ourself so we don't have to wait for the
 * dynamic loading of it.
 * This way the toolbar will have the correct height when the editor is
 * initialized, and the editor will be sized to the correct height.
 */
@import '../../tinymce/skins/white/skin.min.css';
