var resizeLoginBox=function(){function a(){if(b){var a=.4375*c.clientHeight-d.clientHeight/2,e=(c.clientWidth-d.clientWidth)/2;d.style.top=a+"px";d.style.left=e+"px";f.style.width=c.clientWidth+"px";f.style.height=c.clientHeight+"px";f.style.top=-a+"px";f.style.left=-e+"px"}}var b=document.getElementsByTagName("body")[0],f=document.getElementById("bg"),d=document.getElementById("form-container"),c=document.getElementById("loading-mask")||b;b.className+=(0<b.className.length?" ":"")+navigator.sayswho.split(" ")[0]+
" "+navigator.sayswho.replace(" ","");var e=document.createElement("img");e.onload=function(){d.style.visibility="visible"};e.src=window.getComputedStyle(c,!1).backgroundImage.slice(4,-1).replace(/"/g,"");a();window.addEventListener("resize",a);return a}();
